import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import 'package:connectivity/connectivity.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_webview_plugin/flutter_webview_plugin.dart';
import 'package:url_launcher/url_launcher.dart';
import 'ApiMethods.dart';
import 'ShuftiPro.dart';
import 'package:camera/camera.dart';
import 'package:video_player/video_player.dart';
import 'package:quiver/async.dart';
import 'dart:math' as Math;
import 'package:flutter_screenutil/flutter_screenutil.dart';

const MaterialColor white = const MaterialColor(
  0xFFFFFFFF,
  const <int, Color>{
    50: const Color(0xFFFFFFFF),
    100: const Color(0xFFFFFFFF),
    200: const Color(0xFFFFFFFF),
    300: const Color(0xFFFFFFFF),
    400: const Color(0xFFFFFFFF),
    500: const Color(0xFFFFFFFF),
    600: const Color(0xFFFFFFFF),
    700: const Color(0xFFFFFFFF),
    800: const Color(0xFFFFFFFF),
    900: const Color(0xFFFFFFFF),
  },
);

bool isBackside = false,
    backsideFlow = false,
    isWebView = false,
    isBacksideProcessing = false;
Map<String, Object> documentPayloadObj, requestedPayloadTemp;
String supportedTypesObj, videoPathToPass, nationality, WebView_Url = null;
var arr,
    len,
    cameras,
    firstCamera,
    data,
    turn = 1,
    camCount = 1,
    frontCamCountCopy = 0;
List<String> supportedTypes_list = null;
List<String> supportedTypes_list_consent = null;
String consentSupType = "";


int _pointers = 0;
double _minAvailableZoom = 1.0;
double _maxAvailableZoom = 1.0;
double _currentScale = 1.0;
double _baseScale = 1.0;
VideoPlayerController videoController;
XFile imageFile;
XFile videoFile;
VoidCallback videoPlayerListener;


typedef CalbackFunction = void Function(String value);

int cameraDirection, _start = 10, _current = 10, consentSupCounter = 0;
CountdownTimer countDownTimer;

var APIResponse;

Future<bool> check() async {
  var connectivityResult = await (Connectivity().checkConnectivity());
  if (connectivityResult == ConnectivityResult.mobile) {
    return true;
  } else if (connectivityResult == ConnectivityResult.wifi) {
    return true;
  }
  return false;
}

String printVal(str) {
  if (str == '[id_card]' || str.contains('id_card')) {
    return 'ID Card';
  } else if (str == '[credit_or_debit_card]' || str.contains('credit_or_debit_card')) {
    return 'Credit or Debit Card';
  } else if (str == '[passport]' || str.contains('passport')) {
    return 'Passport';
  } else if (str == '[driving_license]' || str.contains('driving_license')) {
    return 'Driving License';
  } else {
    var splitStr = str.toLowerCase().split('_');
    for (var i = 0; i < splitStr.length; i++) {
      if (splitStr[i][0] == '[') {
        splitStr[i] = splitStr[i][1].toUpperCase() + splitStr[i].substring(2);
      } else {
        splitStr[i] = splitStr[i][0].toUpperCase() + splitStr[i].substring(1);
      }
    }
    return splitStr.join(' ');
  }
}

class ShowInstructions extends StatelessWidget {
  var typeList = verificationTypeList.asMap();
  final CalbackFunction callback;

  ShowInstructions({Key key, this.callback}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    camCount = 1;
    isBacksideProcessing = false;
    isShuftiPro = false;
    isPayloadSet = false;

    Widget PrintInstructions() {
      final List<String> face_entries = <String>[
        'Take a photo of your entire face.',
        'Take the photo in proper light and avoid glare.',
        'Remove accessories (glasses, hat, face mask, etc.) before taking photo.'
      ];
      final List<String> doc_entries = <String>[
        'Take a photo of your document in proper light.',
        'Don’t hide or fold any part of your document.',
        'Your entire document should be visible in the photo.',
        'The document information should be readable in the photo.',
        'Avoid glare on your document.'
      ];
      final List<String> address_entries = <String>[
        'Take a photo of your address document in proper light.',
        'Avoid glare on your document.',
        'Entire document should be visible in the photo.',
        'Don’t hide or fold any part of your document.',
        'Address document should not be older than three months.'
      ];
      final List<String> consent_entries = <String>[
        'Take the photo in sufficient light and avoid glare.',
        'Your full face and consent note should be in the photo.',
        'XYZ should be mentioned on the consent note.',
        'Don’t crop, hide or fold any part of the consent note.'
      ];
      if (typeList[0] == "face") {
        len = face_entries.length;
      } else if ((typeList[0] == "document") ||
          (typeList[0] == "document_two")) {
        len = doc_entries.length;
      } else if (typeList[0] == "address") {
        len = address_entries.length;
      } else if (typeList[0] == "consent") {
        requestedPayloadTemp = requestedPayload[typeList[0]];
        if (requestedPayloadTemp.containsKey("supported_types") &&
            requestedPayloadTemp["supported_types"] != null &&
            requestedPayloadTemp["supported_types"] != "") {
          supportedTypes_list_consent = requestedPayloadTemp["supported_types"];
          if (supportedTypes_list_consent.contains("handwritten")) {
            consentSupCounter = consentSupCounter + 1;
            consentSupType = "handwritten";
          }
          if (supportedTypes_list_consent.contains("printed")) {
            consentSupCounter = consentSupCounter + 1;
            consentSupType = "printed";
          }
          if (consentSupCounter == 1) {
            if (consentSupType == "printed") {
              consent_entries.insert(
                  3, "Only printed consent note would be accepted.");
            } else if (consentSupType == "handwritten") {
              consent_entries.insert(
                  3, "Only handwritten consent note would be accepted.");
            }
          }
          consentSupCounter = 0;
          consentSupType = "";
        }
        len = consent_entries.length;
      }
      return Container(
        alignment: Alignment.centerLeft,
        child: Column(
          children: <Widget>[
            ListView.builder(
              shrinkWrap: true,
              itemCount: len,
              itemBuilder: (context, index) {
                return Padding(
                  padding: EdgeInsets.only(
                    bottom: 5.w,
                  ),
                  child: Row(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Text(
                        '${index + 1}  ',
                        style: new TextStyle(
                          fontSize: 14.sp,
                          fontFamily: 'OpenSans',
                          color: Colors.black,
                        ),
                      ),
                      Expanded(
                        child: typeList[0] == "face"
                            ? Text(
                                face_entries[index],
                                style: new TextStyle(
                                  fontSize: 14.sp,
                                  fontFamily: 'OpenSans',
                                  color: Colors.black,
                                ),
                              )
                            : typeList[0] == "document"
                                ? Text(
                                    doc_entries[index],
                                    style: new TextStyle(
                                      fontFamily: 'OpenSans',
                                      fontSize: 14.sp,
                                      color: Colors.black,
                                    ),
                                  )
                                : typeList[0] == "document_two"
                                    ? Text(
                                        doc_entries[index],
                                        style: new TextStyle(
                                          fontFamily: 'OpenSans',
                                          fontSize: 14.sp,
                                          color: Colors.black,
                                        ),
                                      )
                                    : typeList[0] == "address"
                                        ? Text(
                                            address_entries[index],
                                            style: new TextStyle(
                                              fontFamily: 'OpenSans',
                                              fontSize: 14.sp,
                                              color: Colors.black,
                                            ),
                                          )
                                        : typeList[0] == "consent"
                                            ? Text(
                                                consent_entries[index],
                                                style: new TextStyle(
                                                  fontFamily: 'OpenSans',
                                                  fontSize: 14.sp,
                                                  color: Colors.black,
                                                ),
                                              )
                                            : null,
                      ),
                    ],
                  ),
                );
              },
            )
          ],
        ),
      );
    }

    Widget PrintVideoInstructions() {
      final List<String> face_entries = <String>[
        'Record a video of your entire face.',
        'Record a video in proper light and avoid glare.',
        'Remove accessories (glasses, hat, face mask, etc.) before recording the video.'
      ];
      final List<String> doc_entries = <String>[
        'Record a video of your document in proper light.',
        'Don’t hide or fold any part of your document.',
        'Your entire document should be visible in the video.',
        'The document information should be readable in the video.',
        'Avoid glare on your document.'
      ];
      final List<String> address_entries = <String>[
        'Record a clear video of your address document in a proper light.',
        'Document information should be readable in the video.',
        'Entire document should be visible in the video.',
        'Don’t hide or fold any part of your document.',
        'Address document should not be older than three months.'
      ];
      final List<String> consent_entries = <String>[
        'Record a video in sufficient light and avoid glare.',
        'Your full face and consent note should be in the video.',
        'XYZ should be mentioned on the consent note.',
        'Don’t crop, hide or fold any part of the consent note.'
      ];
      if (typeList[0] == "face") {
        len = face_entries.length;
      } else if ((typeList[0] == "document") ||
          (typeList[0] == "document_two")) {
        len = doc_entries.length;
      } else if (typeList[0] == "address") {
        len = address_entries.length;
      } else if (typeList[0] == "consent") {
        requestedPayloadTemp = requestedPayload[typeList[0]];
        if (requestedPayloadTemp.containsKey("supported_types") &&
            requestedPayloadTemp["supported_types"] != null &&
            requestedPayloadTemp["supported_types"] != "") {
          supportedTypes_list_consent = requestedPayloadTemp["supported_types"];
          if (supportedTypes_list_consent.contains("handwritten")) {
            consentSupCounter = consentSupCounter + 1;
            consentSupType = "handwritten";
          }
          if (supportedTypes_list_consent.contains("printed")) {
            consentSupCounter = consentSupCounter + 1;
            consentSupType = "printed";
          }
          if (consentSupCounter == 1) {
            if (consentSupType == "printed") {
              consent_entries.insert(
                  3, "Only printed consent note would be accepted.");
            } else if (consentSupType == "handwritten") {
              consent_entries.insert(
                  3, "Only handwritten consent note would be accepted.");
            }
          }
          consentSupCounter = 0;
          consentSupType = "";
        }
        len = consent_entries.length;
      }
      return Container(
        alignment: Alignment.centerLeft,
        child: Column(
          children: <Widget>[
            ListView.builder(
              shrinkWrap: true,
              itemCount: len,
              itemBuilder: (context, index) {
                return Padding(
                  padding: EdgeInsets.only(
                    bottom: 5.w,
                  ),
                  child: Row(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Text(
                        '${index + 1}  ',
                        style: new TextStyle(
                          fontFamily: 'OpenSans',
                          fontSize: 14.sp,
                          color: Colors.black,
                        ),
                      ),
                      Expanded(
                        child: typeList[0] == "face"
                            ? Text(
                                face_entries[index],
                                style: new TextStyle(
                                  fontFamily: 'OpenSans',
                                  fontSize: 14.sp,
                                  color: Colors.black,
                                ),
                              )
                            : typeList[0] == "document"
                                ? Text(
                                    doc_entries[index],
                                    style: new TextStyle(
                                      fontFamily: 'OpenSans',
                                      fontSize: 14.sp,
                                      color: Colors.black,
                                    ),
                                  )
                                : typeList[0] == "document_two"
                                    ? Text(
                                        doc_entries[index],
                                        style: new TextStyle(
                                          fontFamily: 'OpenSans',
                                          fontSize: 14.sp,
                                          color: Colors.black,
                                        ),
                                      )
                                    : typeList[0] == "address"
                                        ? Text(
                                            address_entries[index],
                                            style: new TextStyle(
                                              fontFamily: 'OpenSans',
                                              fontSize: 14.sp,
                                              color: Colors.black,
                                            ),
                                          )
                                        : typeList[0] == "consent"
                                            ? Text(
                                                consent_entries[index],
                                                style: new TextStyle(
                                                  fontFamily: 'OpenSans',
                                                  fontSize: 14.sp,
                                                  color: Colors.black,
                                                ),
                                              )
                                            : null,
                      ),
                    ],
                  ),
                );
              },
            )
          ],
        ),
      );
    }

    Widget VerificationGif() {
      if (typeList[0] == "face") {
        return Image(
          image: AssetImage("assets/sdk/face.gif", package: 'shufti_test'),
          height: 200.h,
        );
      } else if (typeList[0] == "document" || typeList[0] == "document_two") {
        return Image(
          image: AssetImage("assets/sdk/documentVerification.gif", package: 'shufti_test'),
          height: 200.h,
        );
      } else if (typeList[0] == "address") {
        return Image(
          image: AssetImage("assets/sdk/addressVerification.gif", package: 'shufti_test'),
          height: 200.h,
        );
      } else if (typeList[0] == "consent") {
        return Image(
          image: AssetImage("assets/sdk/consentVerification.gif", package: 'shufti_test'),
          height: 200.h,
        );
      }
    }

    callNextScreen() {
      if ((typeList[0] == "document" ||
              typeList[0] == "document_two" ||
              typeList[0] == "address" ||
              typeList[0] == "consent") &&
          !isBackside) {
        Navigator.push(
          context,
          MaterialPageRoute(
              builder: (context) => DocType(
                    callback: (res) {
                      callback(res);
                      // print("\n\ncalling preview: " +
                      //     res.toString());
                    },
                  )),
        );
      } else {
        Navigator.push(
          context,
          MaterialPageRoute(
              settings: RouteSettings(name: "/Ready"),
              builder: (context) => Ready(
                    callback: (res) {
                      callback(res);
                      // print("\n\ncalling preview: " +
                      //     res.toString());
                    },
                  )),
        );
      }
    }

    Future<bool> _onBackPressed() {
      return showDialog(
            context: context,
            barrierDismissible: false,
            builder: (context) => new AlertDialog(
              title: new Text(
                'Confirm Cancellation',
                style: TextStyle(
                  fontFamily: 'OpenSans',
                ),
              ),
              content: new Text(
                'Are you sure you want to cancel the verification process?',
                style: TextStyle(
                  fontFamily: 'OpenSans',
                ),
              ),
              actions: <Widget>[
                Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: new GestureDetector(
                    onTap: () => Navigator.of(context).pop(false),
                    child: Text(
                      "Not Now",
                      style: TextStyle(
                        fontSize: 13.w,
                        fontFamily: 'OpenSans',
                        color: Color(0xff2b6ad8),
                      ),
                    ),
                  ),
                ),
                SizedBox(height: 16),
                Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: new GestureDetector(
                    onTap: () => {
                      isCheck = false,
                      verificationTypeList = [],
                      indexTempList = [],
                      supportedTypes_list = null,
                      Navigator.pushAndRemoveUntil(
                          context,
                          MaterialPageRoute(builder: (context) => userHome),
                          (route) => false),
                      callback(verificatonCancel),
                    },
                    child: Text(
                      "Confirm",
                      style: TextStyle(
                        fontSize: 13.w,
                        color: Color(0xff2b6ad8),
                        fontFamily: 'OpenSans',
                      ),
                    ),
                  ),
                ),
              ],
            ),
          ) ??
          false;
    }

    String verificationType = printVal(typeList[0]);
    // ScreenUtil.init(new BoxConstraints.loose(Size(360, 690)));
    return WillPopScope(
        onWillPop: _onBackPressed,
        child: Scaffold(
          backgroundColor: Colors.white,
          appBar: AppBar(
            elevation: 0.0,
            centerTitle: true,
            title: Text(
              "$verificationType Verification",
              style: TextStyle(
                color: Colors.black,
                fontFamily: 'OpenSans',
                fontWeight: FontWeight.bold,
              ),
            ),
            backgroundColor: white,
            automaticallyImplyLeading: true,
            leading: GestureDetector(
              onTap: () {
                _onBackPressed();
              },
              child: Container(
                height: 27,
                width: 27,
                alignment: Alignment.centerLeft,
                margin: const EdgeInsets.fromLTRB(20, 0, 0, 0),
                child: Image(
                  image: AssetImage("assets/sdk/back_icon.png", package: 'shufti_test'),
                  height: 20,
                  width: 20,
                ),
              ),
            ),
          ),
          body: Column(
            children: <Widget>[
              Padding(
                padding: EdgeInsets.only(
                  top: 7.w,
                  left: 5.w,
                  bottom: 5.w,
                ),
                child: VerificationGif(),
              ),
              Padding(
                padding: EdgeInsets.only(
                  top: 7.w,
                  left: 35.w,
                  bottom: 5.w,
                ),
                child: Container(
                  alignment: Alignment.centerLeft,
                  child: Text(
                    'Instructions',
                    style: new TextStyle(
                      fontFamily: 'OpenSans',
                      fontSize: 18.sp,
                      fontWeight: FontWeight.w600,
                      color: Colors.black,
                    ),
                  ),
                ),
              ),
              Padding(
                padding: EdgeInsets.only(
                  top: 5.w,
                  left: 25.w,
                  bottom: 5.w,
                  right: 25.w,
                ),
                child: requestedPayload['verification_mode'] == 'image_only'
                    ? PrintInstructions()
                    : PrintVideoInstructions(),
              ),
              Spacer(),
              Padding(
                padding: EdgeInsets.only(
                  left: 10.w,
                  right: 10.w,
                ),
                child: Container(
                  height: 45.h,
                  width: 1.sw,
                  child: RaisedButton(
                    onPressed: () {
                      callNextScreen();
                    },
                    textColor: Colors.white,
                    color: Color(0xff2b6ad8),
                    child: new Container(
                      margin: const EdgeInsets.fromLTRB(5, 0, 0, 0),
                      child: new Text(
                        "Continue",
                        textAlign: TextAlign.center,
                        style: new TextStyle(
                          fontFamily: 'OpenSans',
                          fontSize: 17.sp,
                          fontWeight: FontWeight.bold,
                          color: Colors.white,
                        ),
                      ),
                    ),
                  ),
                ),
              ),
              Spacer(),
              Padding(
                padding: EdgeInsets.only(
                  bottom: 14.w,
                ),
                child: Container(
                    height: 13.h,
                    child: Image.asset("assets/sdk/footerImage.png", package: 'shufti_test')),
              ),
            ],
          ),
        ));
  }
}

class Ready extends StatelessWidget {
  var typeList = verificationTypeList.asMap();
  final CalbackFunction callback;

  Ready({Key key, this.callback}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    if ((typeList[0] == "document" ||
        typeList[0] == "document_two" ||
        typeList[0] == "address" ||
        typeList[0] == "consent")) {
      documentPayloadObj = requestedPayload[typeList[0]];
      supportedTypesObj =
          printVal(documentPayloadObj['supported_types'].toString());
      supportedTypesObj = supportedTypesObj.replaceAll('[', '');
      supportedTypesObj = supportedTypesObj.replaceAll(']', '');
    }

    Widget ImageText() {
      Map<String, Object> reqPassport = requestedPayload[typeList[0]];

      if (nationality == null) {
        nationality = "";
      }
      return Column(
        children: [
          Padding(
              padding: EdgeInsets.only(
                left: 20.w,
                bottom: 5.w,
                right: 20.w,
              ),
              child: typeList[0] == 'face'
                  ? Image(
                      image: AssetImage("assets/sdk/face.png", package: 'shufti_test'),
                      height: 100.h,
                      width: 0.17.sw,
                    )
                  : Image(
                      image: AssetImage("assets/sdk/document.png", package: 'shufti_test'),
                      height: 100.h,
                      width: 0.17.sw,
                    )),
          new Text(
            "Ready to take a photo?",
            style: new TextStyle(
                fontFamily: 'OpenSans',
                fontSize: 20.sp,
                color: Colors.black,
                fontWeight: FontWeight.bold),
          ),
          Padding(
            padding: EdgeInsets.only(
              top: 13.w,
              left: 30.w,
              bottom: 5.w,
              right: 30.w,
            ),
            child: typeList[0] == 'face'
                ? new Text(
                    "Take a photo of your entire face",
                    textAlign: TextAlign.center,
                    style: new TextStyle(
                      fontFamily: 'OpenSans',
                      fontSize: 16.sp,
                      color: Colors.black,
                      fontWeight: FontWeight.w600,
                    ),
                  )
                : typeList[0] == 'consent'
                    ? new Text(
                        "Take a photo of the consent note along with your face",
                        textAlign: TextAlign.center,
                        style: new TextStyle(
                          fontFamily: 'OpenSans',
                          fontSize: 16.sp,
                          color: Colors.black,
                          fontWeight: FontWeight.w600,
                        ),
                      )
                    : typeList[0] == 'address'
                        ? reqPassport['supported_types']
                                .toString()
                                .contains("passport")
                            ? new Text(
                                "Take a picture of the photo page of your $nationality $supportedTypesObj",
                                textAlign: TextAlign.center,
                                style: new TextStyle(
                                  fontFamily: 'OpenSans',
                                  fontSize: 16.sp,
                                  color: Colors.black,
                                  fontWeight: FontWeight.w600,
                                ),
                              )
                            : new Text(
                                "Take a photo of your $nationality $supportedTypesObj",
                                textAlign: TextAlign.center,
                                style: new TextStyle(
                                  fontFamily: 'OpenSans',
                                  fontSize: 16.sp,
                                  color: Colors.black,
                                  fontWeight: FontWeight.w600,
                                ),
                              )
                        : !isBackside
                            ? reqPassport['supported_types']
                                    .toString()
                                    .contains("passport")
                                ? new Text(
                                    "Take a picture of the photo page of your $nationality $supportedTypesObj",
                                    textAlign: TextAlign.center,
                                    style: new TextStyle(
                                      fontFamily: 'OpenSans',
                                      fontSize: 16.sp,
                                      color: Colors.black,
                                      fontWeight: FontWeight.w600,
                                    ),
                                  )
                                : new Text(
                                    "Take a photo of the front side of your $nationality $supportedTypesObj",
                                    textAlign: TextAlign.center,
                                    style: new TextStyle(
                                      fontFamily: 'OpenSans',
                                      fontSize: 16.sp,
                                      color: Colors.black,
                                      fontWeight: FontWeight.w600,
                                    ),
                                  )
                            : reqPassport['supported_types']
                                    .toString()
                                    .contains("passport")
                                ? new Text(
                                    "Take a picture of the second page of your $nationality $supportedTypesObj",
                                    textAlign: TextAlign.center,
                                    style: new TextStyle(
                                      fontFamily: 'OpenSans',
                                      fontSize: 16.sp,
                                      color: Colors.black,
                                      fontWeight: FontWeight.w600,
                                    ),
                                  )
                                : new Text(
                                    "Take a photo of the backside of your $nationality $supportedTypesObj",
                                    textAlign: TextAlign.center,
                                    style: new TextStyle(
                                      fontFamily: 'OpenSans',
                                      fontSize: 16.sp,
                                      color: Colors.black,
                                      fontWeight: FontWeight.w600,
                                    ),
                                  ),
          ),
          Spacer(),
        ],
      );
    }

    Widget VideoText() {
      Map<String, Object> reqPassport = requestedPayload[typeList[0]];

      if (nationality == null) {
        nationality = "";
      }
      return Column(
        children: [
          Padding(
            padding: EdgeInsets.only(
              left: 20.w,
              bottom: 5.w,
              right: 20.w,
            ),
            child: requestedPayload.containsKey('face')
                ? Image(
                    image: AssetImage("assets/sdk/face.png", package: 'shufti_test'),
                    height: 100.h,
                    width: 0.17.sw,
                  )
                : Image(
                    image: AssetImage("assets/sdk/document.png", package: 'shufti_test'),
                    height: 100.h,
                    width: 0.17.sw,
                  ),
          ),
          new Text(
            "Ready to record a video?",
            style: new TextStyle(
                fontFamily: 'OpenSans',
                fontSize: 20.sp,
                color: Colors.black,
                fontWeight: FontWeight.bold),
          ),
          Padding(
            padding: EdgeInsets.only(
              top: 13.w,
              left: 30.w,
              bottom: 5.w,
              right: 30.w,
            ),
            child: requestedPayload.containsKey("face") &&
                    verificationTypeList[0] == "face"
                ? new Text(
                    "Record a video of your entire face",
                    textAlign: TextAlign.center,
                    style: new TextStyle(
                      fontFamily: 'OpenSans',
                      fontSize: 16.sp,
                      color: Colors.black,
                      fontWeight: FontWeight.w600,
                    ),
                  )
                : requestedPayload.containsKey("consent") &&
                        verificationTypeList[0] == "consent"
                    ? new Text(
                        "Record a video of the consent note with your face",
                        textAlign: TextAlign.center,
                        style: new TextStyle(
                          fontFamily: 'OpenSans',
                          fontSize: 16.sp,
                          color: Colors.black,
                          fontWeight: FontWeight.w600,
                        ),
                      )
                    : typeList[0] == 'address'
                        ? reqPassport['supported_types']
                                .toString()
                                .contains("passport")
                            ? new Text(
                                "Record a video of the photo page of your $nationality $supportedTypesObj",
                                textAlign: TextAlign.center,
                                style: new TextStyle(
                                  fontFamily: 'OpenSans',
                                  fontSize: 16.sp,
                                  color: Colors.black,
                                  fontWeight: FontWeight.w600,
                                ),
                              )
                            : new Text(
                                "Record a video of your $nationality $supportedTypesObj",
                                textAlign: TextAlign.center,
                                style: new TextStyle(
                                  fontFamily: 'OpenSans',
                                  fontSize: 16.sp,
                                  color: Colors.black,
                                  fontWeight: FontWeight.w600,
                                ),
                              )
                        : !isBackside
                            ? reqPassport['supported_types']
                                    .toString()
                                    .contains("passport")
                                ? new Text(
                                    "Record a video of the photo page of your $nationality $supportedTypesObj",
                                    textAlign: TextAlign.center,
                                    style: new TextStyle(
                                      fontFamily: 'OpenSans',
                                      fontSize: 16.sp,
                                      color: Colors.black,
                                      fontWeight: FontWeight.w600,
                                    ),
                                  )
                                : new Text(
                                    "Record a video of the front side of your $nationality $supportedTypesObj",
                                    textAlign: TextAlign.center,
                                    style: new TextStyle(
                                      fontFamily: 'OpenSans',
                                      fontSize: 16.sp,
                                      color: Colors.black,
                                      fontWeight: FontWeight.w600,
                                    ),
                                  )
                            : reqPassport['supported_types']
                                    .toString()
                                    .contains("passport")
                                ? new Text(
                                    "Record a video of the second page of your $nationality $supportedTypesObj",
                                    textAlign: TextAlign.center,
                                    style: new TextStyle(
                                      fontFamily: 'OpenSans',
                                      fontSize: 16.sp,
                                      color: Colors.black,
                                      fontWeight: FontWeight.w600,
                                    ),
                                  )
                                : new Text(
                                    "Record a video of the backside of your $nationality $supportedTypesObj",
                                    textAlign: TextAlign.center,
                                    style: new TextStyle(
                                      fontFamily: 'OpenSans',
                                      fontSize: 16.sp,
                                      color: Colors.black,
                                      fontWeight: FontWeight.w600,
                                    ),
                                  ),
          ),
          Spacer(),
        ],
      );
    }

    // Widget ImageText() {
    //   if (nationality == null) {
    //     nationality = "";
    //   }
    //   return Column(
    //     children: [
    //       Padding(
    //           padding: EdgeInsets.only(
    //             left: 20.w,
    //             bottom: 5.w,
    //             right: 20.w,
    //           ),
    //           child: typeList[0] == 'face'
    //               ? Image(
    //                   image: AssetImage("assets/sdk/face.png"),
    //                   height: 100.h,
    //                   width: 0.17.sw,
    //                 )
    //               : Image(
    //                   image: AssetImage("assets/sdk/document.png"),
    //                   height: 100.h,
    //                   width: 0.17.sw,
    //                 )),
    //       new Text(
    //         "Ready to take photo? ",
    //         style: new TextStyle(
    //             fontFamily: 'OpenSans',
    //             fontSize: 20.sp,
    //             color: Colors.black,
    //             fontWeight: FontWeight.bold),
    //       ),
    //       Padding(
    //         padding: EdgeInsets.only(
    //           top: 13.w,
    //           left: 30.w,
    //           bottom: 5.w,
    //           right: 30.w,
    //         ),
    //         child: typeList[0] == 'face'
    //             ? new Text(
    //                 "Take a photo of your entire face. ",
    //                 style: new TextStyle(
    //                   fontFamily: 'OpenSans',
    //                   fontSize: 16.sp,
    //                   color: Colors.black,
    //                   fontWeight: FontWeight.w600,
    //                 ),
    //               )
    //             : typeList[0] == 'consent'
    //                 ? new Text(
    //                     "Take a photo of your $supportedTypesObj consent. ",
    //                     style: new TextStyle(
    //                       fontFamily: 'OpenSans',
    //                       fontSize: 16.sp,
    //                       color: Colors.black,
    //                       fontWeight: FontWeight.w600,
    //                     ),
    //                   )
    //                 : typeList[0] == 'address'
    //                     ? new Text(
    //                         "Take a photo of your $nationality $supportedTypesObj. ",
    //                         style: new TextStyle(
    //                           fontFamily: 'OpenSans',
    //                           fontSize: 16.sp,
    //                           color: Colors.black,
    //                           fontWeight: FontWeight.w600,
    //                         ),
    //                       )
    //                     : !isBackside
    //                         ? new Text(
    //                             "Take the frontside photo of your $nationality $supportedTypesObj.",
    //                             textAlign: TextAlign.center,
    //                             style: new TextStyle(
    //                               fontFamily: 'OpenSans',
    //                               fontSize: 16.sp,
    //                               color: Colors.black,
    //                               fontWeight: FontWeight.w600,
    //                             ),
    //                           )
    //                         : new Text(
    //                             "Take the backside photo of your $nationality $supportedTypesObj.",
    //                             textAlign: TextAlign.center,
    //                             style: new TextStyle(
    //                               fontFamily: 'OpenSans',
    //                               fontSize: 16.sp,
    //                               color: Colors.black,
    //                               fontWeight: FontWeight.w600,
    //                             ),
    //                           ),
    //       ),
    //       Spacer(),
    //     ],
    //   );
    // }
    //
    // Widget VideoText() {
    //   if (nationality == null) {
    //     nationality = "";
    //   }
    //   return Column(
    //     children: [
    //       Padding(
    //         padding: EdgeInsets.only(
    //           left: 20.w,
    //           bottom: 5.w,
    //           right: 20.w,
    //         ),
    //         child: requestedPayload.containsKey('face')
    //             ? Image(
    //                 image: AssetImage("assets/sdk/face.png"),
    //                 height: 100.h,
    //                 width: 0.17.sw,
    //               )
    //             : Image(
    //                 image: AssetImage("assets/sdk/document.png"),
    //                 height: 100.h,
    //                 width: 0.17.sw,
    //               ),
    //       ),
    //       new Text(
    //         "Ready to record video? ",
    //         style: new TextStyle(
    //             fontFamily: 'OpenSans',
    //             fontSize: 20.sp,
    //             color: Colors.black,
    //             fontWeight: FontWeight.bold),
    //       ),
    //       Padding(
    //         padding: EdgeInsets.only(
    //           top: 13.w,
    //           left: 30.w,
    //           bottom: 5.w,
    //           right: 30.w,
    //         ),
    //         child: requestedPayload.containsKey("face") &&
    //                 verificationTypeList[0] == "face"
    //             ? new Text(
    //                 "Record video of your entire face. ",
    //                 style: new TextStyle(
    //                   fontFamily: 'OpenSans',
    //                   fontSize: 16.sp,
    //                   color: Colors.black,
    //                   fontWeight: FontWeight.w600,
    //                 ),
    //               )
    //             : requestedPayload.containsKey("consent") &&
    //                     verificationTypeList[0] == "consent"
    //                 ? new Text(
    //                     "Record video of your $supportedTypesObj consent. ",
    //                     style: new TextStyle(
    //                       fontFamily: 'OpenSans',
    //                       fontSize: 16.sp,
    //                       color: Colors.black,
    //                       fontWeight: FontWeight.w600,
    //                     ),
    //                   )
    //                 : typeList[0] == 'address'
    //                     ? new Text(
    //                         "Record the video of your $nationality $supportedTypesObj. ",
    //                         style: new TextStyle(
    //                           fontFamily: 'OpenSans',
    //                           fontSize: 16.sp,
    //                           color: Colors.black,
    //                           fontWeight: FontWeight.w600,
    //                         ),
    //                       )
    //                     : !isBackside
    //                         ? new Text(
    //                             "Record the frontside video of your $nationality $supportedTypesObj.",
    //                             textAlign: TextAlign.center,
    //                             style: new TextStyle(
    //                               fontFamily: 'OpenSans',
    //                               fontSize: 16.sp,
    //                               color: Colors.black,
    //                               fontWeight: FontWeight.w600,
    //                             ),
    //                           )
    //                         : new Text(
    //                             "Record the backside video of your $nationality $supportedTypesObj.",
    //                             textAlign: TextAlign.center,
    //                             style: new TextStyle(
    //                               fontFamily: 'OpenSans',
    //                               fontSize: 16.sp,
    //                               color: Colors.black,
    //                               fontWeight: FontWeight.w600,
    //                             ),
    //                           ),
    //       ),
    //       Spacer(),
    //     ],
    //   );
    // }

    Future<bool> _onBackPressed() {
      if (isBackside && isBacksideProcessing) {
        camCount = frontCamCountCopy;
        isBacksideProcessing = false;
      }
      isBackside = false;
      turn = 2;
      Navigator.of(context).pop();
    }

    String verificationType = printVal(typeList[0]);

    BoxDecoration myBoxDecoration() {
      return BoxDecoration(
        border: Border.all(
          color: Colors.black12,
          width: 0.9,
        ),
      );
    }

    // ScreenUtil.init(new BoxConstraints.loose(Size(360, 690)));

    return WillPopScope(
        onWillPop: _onBackPressed,
        child: Scaffold(
          backgroundColor: Colors.white,
          appBar: AppBar(
            elevation: 0.0,
            centerTitle: true,
            title: Text(
              "$verificationType Verification",
              style: TextStyle(
                color: Colors.black,
                fontFamily: 'OpenSans',
                fontWeight: FontWeight.bold,
              ),
            ),
            backgroundColor: white,
            automaticallyImplyLeading: true,
            leading: GestureDetector(
              onTap: () {
                _onBackPressed();
              },
              child: Container(
                height: 27,
                width: 27,
                alignment: Alignment.centerLeft,
                margin: const EdgeInsets.fromLTRB(20, 0, 0, 0),
                child: Image(
                  image: AssetImage("assets/sdk/back_icon.png", package: 'shufti_test'),
                  height: 20,
                  width: 20,
                ),
              ),
            ),
          ),
          body: Center(
            child: Column(
              children: <Widget>[
                Spacer(),
                Expanded(
                  flex: 2,
                  child: requestedPayload['verification_mode'] == 'image_only'
                      ? ImageText()
                      : VideoText(),
                ),
                Padding(
                  padding: EdgeInsets.only(
                    top: 7.w,
                    left: 10.w,
                    bottom: 5.w,
                    right: 10.w,
                  ),
                  child: Container(
                    height: 45.h,
                    width: 1.sw,
                    child: OutlinedButton(
                      onPressed: () {
                        _current = 10;
                        turn = 1;
                        if (isBackside) {
                          frontCamCountCopy = camCount;
                          isBacksideProcessing = true;
                        }
                        camCount = 1;
                        requestedPayload['verification_mode'] == 'image_only'
                            ? Navigator.push(
                                context,
                                MaterialPageRoute(
                                    builder: (context) => ImageCapture(
                                          callback: (res) {
                                            callback(res);
                                            // print("\n\ncalling preview: " +
                                            //     res.toString());
                                          },
                                        )),
                              )
                            : Navigator.push(
                                context,
                                MaterialPageRoute(
                                    builder: (context) => VideoRecorder(
                                          callback: (res) {
                                            callback(res);
                                            // print("\n\ncalling preview: " +
                                            //     res.toString());
                                          },
                                        )),
                              );
                      },
                      style: OutlinedButton.styleFrom(
                        backgroundColor: Color(0xff2b6ad8),
                      ),
                      child: new Text(
                        "Yes, I'm Ready",
                        textAlign: TextAlign.center,
                        style: new TextStyle(
                          fontFamily: 'OpenSans',
                          fontSize: 17.sp,
                          fontWeight: FontWeight.bold,
                          color: Colors.white,
                        ),
                      ),
                    ),
                  ),
                ),
                typeList[0] == "document" || typeList[0] == "document_two"
                    ? requestedPayloadTemp
                                .containsKey("backside_proof_required") &&
                            (requestedPayloadTemp["backside_proof_required"] ==
                                    "0" ||
                                (requestedPayloadTemp["supported_types"]
                                            .toString()
                                            .toLowerCase() ==
                                        "[passport]" &&
                                    requestedPayloadTemp[
                                            "backside_proof_required"] ==
                                        "1")) &&
                            isBackside
                        ? Padding(
                            padding: EdgeInsets.only(
                              top: 7.w,
                              left: 10.w,
                              bottom: 5.w,
                              right: 10.w,
                            ),
                            child: Container(
                              height: 45.h,
                              width: 1.sw,
                              decoration: myBoxDecoration(),
                              child: OutlinedButton(
                                onPressed: () {
                                  verificationTypeList.removeAt(0);
                                  supportedTypes_list = null;
                                  typeList = verificationTypeList.asMap();
                                  isBackside = false;
                                  if (verificationTypeList.length == 0) {
                                    if (asyncSDK &&
                                        !requestedPayload
                                            .containsKey('background_checks') &&
                                        !requestedPayload
                                            .containsKey('phone')) {
                                      check().then((intenet) {
                                        if (intenet != null && intenet) {
                                          ApiMethods()
                                              .apiRequest()
                                              .then((response) {
                                            callback(response);
                                          });
                                        } else {
                                          callback(
                                              "internet.connection.problem");
                                        }
                                        isCheck = false;
                                        Navigator.pushAndRemoveUntil(
                                            context,
                                            MaterialPageRoute(
                                                builder: (context) => userHome),
                                            (route) => false);
                                        callback(
                                            "event: request.pending, error: '', reference: " +
                                                createdPayloadSDK["reference"]);
                                      });
                                    } else {
                                      Navigator.push(
                                        context,
                                        MaterialPageRoute(
                                            builder: (context) => Result(
                                                  callback: (res) {
                                                    callback(res);
                                                    // print("\n\ncalling preview: " +
                                                    //     res.toString());
                                                  },
                                                )),
                                      );
                                    }
                                  } else {
                                    Navigator.push(
                                      context,
                                      MaterialPageRoute(
                                          builder: (context) =>
                                              ShowInstructions(
                                                callback: (res) {
                                                  callback(res);
                                                  // print("\n\ncalling preview: " +
                                                  //     res.toString());
                                                },
                                              )),
                                    );
                                  }
                                },
                                style: OutlinedButton.styleFrom(
                                  backgroundColor: Colors.white,
                                ),
                                child: new Text(
                                  "Skip",
                                  textAlign: TextAlign.center,
                                  style: new TextStyle(
                                    fontFamily: 'OpenSans',
                                    fontSize: 17.sp,
                                    fontWeight: FontWeight.bold,
                                    color: Colors.black,
                                  ),
                                ),
                              ),
                            ),
                          )
                        : new Text("")
                    : new Text(""),
                Spacer(),
                Padding(
                  padding: EdgeInsets.only(
                    bottom: 14.w,
                  ),
                  child: Container(
                      height: 13.h,
                      child: Image.asset("assets/sdk/footerImage.png", package: 'shufti_test')),
                ),
              ],
            ),
          ),
        ));
  }
}

class Result extends StatefulWidget {
  final CalbackFunction callback;

  Result({Key key, this.title, this.callback}) : super(key: key);
  final String title;

  @override
  ResultScreen createState() => ResultScreen();
}

class ResultScreen extends State<Result> {
  bool isLoading = true;
  String event = null;

  int show_result = requestedPayload['show_results'];
  int show_privacy_policy = requestedPayload['show_privacy_policy'];

  void setLoading(bool newVal) => setState(() {
        isLoading = newVal;
      });

  void setEvent(String newVal) => setState(() {
        event = newVal;
      });

  void setWebview(bool newVal) => setState(() {
        isWebView = newVal;
      });

  void setVerificationUrl(String newVal) => setState(() {
        WebView_Url = newVal;
      });
  final flutterWebviewPlugin = new FlutterWebviewPlugin();

  @override
  void initState() {
    super.initState();
    flutterWebviewPlugin.onUrlChanged.listen((String url) {
      //String Reference = requestedPayload["reference"];
      if (url.contains("example.com")) {
        print("Dummy Url");
        widget.callback("event: request.pending, error: '', reference: " +
            createdPayloadSDK["reference"]);
        isCheck = false;
        Navigator.pushAndRemoveUntil(
            context,
            MaterialPageRoute(builder: (context) => userHome),
            (route) => false);
        ApiMethods().statusApiRequest().then((response) {
          widget.callback(response);
        });
      }
    });
  }

  Future<bool> _onBackPressed() {
    return showDialog(
          context: context,
          barrierDismissible: false,
          builder: (context) => new AlertDialog(
            title: new Text(
              'Confirm Cancellation',
              style: TextStyle(
                fontFamily: 'OpenSans',
              ),
            ),
            content: new Text(
              'Are you sure you want to cancel the verification process?',
              style: TextStyle(
                fontFamily: 'OpenSans',
              ),
            ),
            actions: <Widget>[
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: new GestureDetector(
                  onTap: () => Navigator.of(context).pop(false),
                  child: Text(
                    "Not Now",
                    style: TextStyle(
                      fontSize: 13.w,
                      color: Color(0xff2b6ad8),
                      fontFamily: 'OpenSans',
                    ),
                  ),
                ),
              ),
              SizedBox(height: 16),
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: new GestureDetector(
                  onTap: () => {
                    isCheck = false,
                    verificationTypeList = [],
                    indexTempList = [],
                    supportedTypes_list = null,
                    widget.callback(verificatonCancel),
                    Navigator.pushAndRemoveUntil(
                        context,
                        MaterialPageRoute(builder: (context) => userHome),
                        (route) => false)
                  },
                  child: Text(
                    "Confirm",
                    style: TextStyle(
                      fontSize: 13.w,
                      color: Color(0xff2b6ad8),
                      fontFamily: 'OpenSans',
                    ),
                  ),
                ),
              ),
            ],
          ),
        ) ??
        false;
  }

  Future<bool> webViewBackpress() {
    flutterWebviewPlugin.hide();
    return showDialog(
      context: context,
      barrierDismissible: false,
      builder: (context) => new AlertDialog(
        title: new Text(
          'Confirm Cancellation',
          style: TextStyle(
            fontFamily: 'OpenSans',
          ),
        ),
        content: new Text(
          'Are you sure you want to cancel the verification process?',
          style: TextStyle(
            fontFamily: 'OpenSans',
          ),
        ),
        actions: <Widget>[
          Padding(
            padding: const EdgeInsets.all(8.0),
            child: new GestureDetector(
              onTap: () => {
                flutterWebviewPlugin.show(),
                Navigator.of(context).pop(false)
              },
              child: Text(
                "Not Now",
                style: TextStyle(
                  fontSize: 13.w,
                  color: Color(0xff2b6ad8),
                  fontFamily: 'OpenSans',
                ),
              ),
            ),
          ),
          SizedBox(height: 16),
          Padding(
            padding: const EdgeInsets.all(8.0),
            child: new GestureDetector(
              onTap: () => {
                isCheck = false,
                verificationTypeList = [],
                indexTempList = [],
                supportedTypes_list = null,
                Navigator.pushAndRemoveUntil(
                    context,
                    MaterialPageRoute(builder: (context) => userHome),
                    (route) => false),
                flutterWebviewPlugin.dispose(),
                widget.callback(verificatonCancel),
              },
              child: Text(
                "Confirm",
                style: TextStyle(
                  fontSize: 13.w,
                  color: Color(0xff2b6ad8),
                  fontFamily: 'OpenSans',
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    // ScreenUtil.init(new BoxConstraints.loose(Size(360, 690)));
    if (isLoading) {
      check().then((intenet) {
        if (intenet != null && intenet) {
          ApiMethods().apiRequest().then((response) {
            APIResponse = response;
            if (response.length > 15) {
              final Responce = json.decode(response);
              setEvent(Responce['event']);
              if(response.toLowerCase().contains('gateway timeout')){
                setEvent('request.timeout');
              }

              if (Responce['event'] == "request.pending") {
                setVerificationUrl(Responce['verification_url']);
                setWebview(true);
              }
              setLoading(false);
            } else {
              setEvent(response);
              setLoading(false);
            }
          });
        } else {
          setEvent("internet.connection");
          setLoading(false);
        }
      });
    }

    return isLoading
        ? WillPopScope(
            onWillPop: _onBackPressed,
            child: Scaffold(
                backgroundColor: Colors.white,
                body: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    Padding(
                      padding: EdgeInsets.only(
                        top: 60.w,
                        left: 15.w,
                        right: 15.w,
                      ),
                      child: Text(
                        "Verification",
                        style: TextStyle(
                          color: Colors.black,
                          fontFamily: 'OpenSans',
                          fontSize: 20.sp,
                          fontWeight: FontWeight.bold,
                        ),
                      ),
                    ),
                    Spacer(),
                    Container(
                      alignment: Alignment.center,
                      child: Column(
                        mainAxisAlignment: MainAxisAlignment.center,
                        children: [
                          Image.asset(
                            "assets/sdk/Preload_Animation.gif", package: 'shufti_test',
                            height: 100.h,
                            width: 0.4.sw,
                          ),
                          Padding(
                            padding: EdgeInsets.only(
                              top: 25.w,
                              left: 25.w,
                              bottom: 5.w,
                              right: 25.w,
                            ),
                            child: Text(
                              "Verifying your identity",
                              textAlign: TextAlign.center,
                              style: TextStyle(
                                  fontFamily: 'OpenSans', fontSize: 18.sp),
                            ),
                          ),
                          Padding(
                            padding: EdgeInsets.only(
                              top: 1.w,
                              left: 25.w,
                              bottom: 5.w,
                              right: 25.w,
                            ),
                            child: Text(
                              "Please Wait...",
                              textAlign: TextAlign.center,
                              style: TextStyle(
                                  fontFamily: 'OpenSans',
                                  fontSize: 18.sp,
                                  fontWeight: FontWeight.bold),
                            ),
                          )
                        ],
                      ),
                    ),
                    Spacer(),
                    Padding(
                      padding: EdgeInsets.only(
                        bottom: 14.w,
                      ),
                      child: Container(
                          height: 13.h,
                          child: Image.asset("assets/sdk/footerImage.png", package: 'shufti_test')),
                    ),
                  ],
                )))
        : isWebView
            ? WillPopScope(
                onWillPop: () {
                  webViewBackpress();
                },
                child: Scaffold(
                  backgroundColor: Colors.white,
                  appBar: AppBar(
                    backgroundColor: white,
                    elevation: 0.0,
                    automaticallyImplyLeading: true,
                    leading: GestureDetector(
                      onTap: () {
                        webViewBackpress();
                      },
                      child: Container(
                        height: 27,
                        width: 27,
                        alignment: Alignment.centerLeft,
                        margin: const EdgeInsets.fromLTRB(20, 0, 0, 0),
                        child: Image(
                          image: AssetImage("assets/sdk/back_icon.png", package: 'shufti_test'),
                          height: 20,
                          width: 20,
                        ),
                      ),
                    ),
                  ),
                  body: Container(
                    height: 900.h,
                    width: 1.sw,
                    alignment: Alignment.center,
                    child: new MaterialApp(
                      routes: {
                        "/": (_) => SafeArea(
                              child: Padding(
                                padding: EdgeInsets.only(
                                  top: 40.w,
                                  left: 0.w,
                                  bottom: 0.w,
                                  right: 0.w,
                                ),
                                child: new WebviewScaffold(
                                  url: WebView_Url,
                                  // initialChild: Container(
                                  //   child: Center(
                                  //       child: Column(
                                  //         mainAxisAlignment: MainAxisAlignment.center,
                                  //         children: [
                                  //           Padding(
                                  //             padding: EdgeInsets.only(
                                  //               top: 60.w,
                                  //               left: 15.w,
                                  //               right: 15.w,
                                  //             ),
                                  //             child: Text(
                                  //               "Verification",
                                  //               style: TextStyle(
                                  //                 color: Colors.black,
                                  //                 fontFamily: 'OpenSans',
                                  //                 fontSize: 20.sp,
                                  //                 fontWeight: FontWeight.bold,
                                  //               ),
                                  //             ),
                                  //           ),
                                  //           Spacer(),
                                  //           Container(
                                  //             alignment: Alignment.center,
                                  //             child: Column(
                                  //               mainAxisAlignment: MainAxisAlignment.center,
                                  //               children: [
                                  //                 Image.asset(
                                  //                   "assets/sdk/Preload_Animation.gif",
                                  //                   height: 100.h,
                                  //                   width: 0.4.sw,
                                  //                 ),
                                  //                 Padding(
                                  //                   padding: EdgeInsets.only(
                                  //                     top: 25.w,
                                  //                     left: 25.w,
                                  //                     bottom: 5.w,
                                  //                     right: 25.w,
                                  //                   ),
                                  //                   child: Text(
                                  //                     "Verifying your identity",
                                  //                     textAlign: TextAlign.center,
                                  //                     style: TextStyle(
                                  //                         fontFamily: 'OpenSans', fontSize: 18.sp),
                                  //                   ),
                                  //                 ),
                                  //                 Padding(
                                  //                   padding: EdgeInsets.only(
                                  //                     top: 1.w,
                                  //                     left: 25.w,
                                  //                     bottom: 5.w,
                                  //                     right: 25.w,
                                  //                   ),
                                  //                   child: Text("Please Wait...",
                                  //                     textAlign: TextAlign.center,
                                  //                     style: TextStyle(
                                  //                         fontFamily: 'OpenSans',
                                  //                         fontSize: 18.sp,
                                  //                         fontWeight: FontWeight.bold),
                                  //                   ),
                                  //                 )
                                  //               ],
                                  //             ),
                                  //           ),
                                  //           Spacer(),
                                  //           Padding(
                                  //             padding: EdgeInsets.only(
                                  //               bottom: 9.w,
                                  //             ),
                                  //             child: Container(
                                  //                 height: 13.h,
                                  //                 child: Image.asset("assets/sdk/footerImage.png")),
                                  //           ),
                                  //         ],
                                  //       )
                                  //   ),
                                  // ),
                                ),
                              ),
                            ),
                      },
                    ),
                  ),
                ),
              )
            : WillPopScope(
                onWillPop: _onBackPressed,
                child: Scaffold(
                  backgroundColor: Colors.white,
                  body: Column(
                    children: <Widget>[
                      Padding(
                        padding: EdgeInsets.only(
                          top: 55.w,
                          left: 25.w,
                          right: 25.w,
                        ),
                        child: Text(
                          "Verification Results",
                          style: TextStyle(
                              fontFamily: 'OpenSans',
                              fontSize: 20.sp,
                              fontWeight: FontWeight.bold),
                        ),
                      ),
                      Padding(
                        padding: EdgeInsets.only(
                          top: 50.w,
                          left: 25.w,
                          bottom: 50.w,
                          right: 25.w,
                        ),
                        child: Center(
                          child: Column(
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              Text(
                                event == 'verification.accepted'
                                    ? "Verified"
                                    : event == 'verification.declined'
                                        ? "Verification Unsuccessful"
                                        : event == 'request.unauthorized'
                                            ? "Unauthorized Request"
                                            : event == 'request.invalid'
                                                ? "Invalid Request"
                                                : event == 'request.timeout'
                                                    ? "Request Timeout"
                                                    : event ==
                                                            'internet.connection'
                                                        ? "No Internet Connection"
                                                        : event ==
                                                                'request.received'
                                                            ? "Verification Completed"
                                                            : "Error",
                                style: TextStyle(
                                    fontFamily: 'OpenSans',
                                    fontSize: 22.sp,
                                    color: event == 'verification.accepted' || event == 'request.received'
                                        ? Colors.green
                                        : Colors.red,
                                    fontWeight: FontWeight.bold),
                              ),
                              Padding(
                                padding: event == 'internet.connection'
                                    ? EdgeInsets.only(
                                        top: 8.w,
                                        left: 5.w,
                                        right: 5.w,
                                      )
                                    : EdgeInsets.only(
                                        top: 10.w,
                                        left: 25.w,
                                        right: 25.w,
                                      ),
                                child: Text(
                                  event == 'verification.accepted'
                                      ? "Your identity is verified"
                                      : event == 'verification.declined'
                                          ? "Sorry, Your identity is not verified"
                                          : event == 'request.unauthorized'
                                              ? "Authorization keys are invalid/missing"
                                              : event == 'request.invalid'
                                                  ? "Your request has been invalid"
                                                  : event == 'request.timeout'
                                                      ? "Your request has timed out"
                                                      : event ==
                                                              'internet.connection'
                                                          ? "Make sure your device is connected to the internet"
                                                          : event ==
                                                                  'request.received'
                                                              ? "Verification results will be forwarded to your merchant"
                                                              : "Error",
                                  textAlign: TextAlign.center,
                                  style: TextStyle(
                                    fontFamily: 'OpenSans',
                                    fontSize: 15.sp,
                                  ),
                                ),
                              ),
                              Padding(
                                padding: EdgeInsets.only(
                                  top: 45.w,
                                  bottom: 45.w,
                                ),
                                child: Image.asset(
                                  event == 'verification.accepted'
                                      ? "assets/sdk/tick.gif"
                                      : event == 'verification.declined'
                                          ? "assets/sdk/cross.gif"
                                          : event == 'request.unauthorized'
                                              ? "assets/sdk/warning.gif"
                                              : event == 'request.invalid'
                                                  ? "assets/sdk/warning.gif"
                                                  : event == 'request.timeout'
                                                      ? "assets/sdk/timeout.gif"
                                                      : event ==
                                                              'internet.connection'
                                                          ? "assets/sdk/cross.gif"
                                                          : event ==
                                                                  'request.received'
                                                              ? "assets/sdk/like_3.gif"
                                                              : "assets/sdk/cross.gif",
                                  height: 100.h,
                                  width: 0.4.sw,
                                  package: 'shufti_test',
                                ),
                              ),
                              show_privacy_policy == 1
                                  ? Padding(
                                      padding: event == 'internet.connection'
                                          ? EdgeInsets.only(
                                              top: 18.w,
                                              left: 15.w,
                                              bottom: 40.w,
                                              right: 15.w,
                                            )
                                          : EdgeInsets.only(
                                              top: 20.w,
                                              left: 15.w,
                                              bottom: 45.w,
                                              right: 15.w,
                                            ),
                                      child: RichText(
                                        text: new TextSpan(
                                          children: [
                                            new TextSpan(
                                              text:
                                                  "Shufti Pro acknowledges your right to request access or erasure of your data. You may review our ",
                                              style: new TextStyle(
                                                fontFamily: 'OpenSans',
                                                fontSize: 15.sp,
                                                fontWeight: FontWeight.normal,
                                                color: Colors.black,
                                              ),
                                            ),
                                            new TextSpan(
                                              text: 'Privacy Policy',
                                              style: new TextStyle(
                                                fontFamily: 'OpenSans',
                                                fontSize: 15.sp,
                                                fontWeight: FontWeight.normal,
                                                decoration:
                                                    TextDecoration.underline,
                                                color: Color(0xff2b6ad8),
                                              ),
                                              recognizer:
                                                  new TapGestureRecognizer()
                                                    ..onTap = () {
                                                      launch(
                                                          'https://shuftipro.com/privacy-policy');
                                                    },
                                            ),
                                            new TextSpan(
                                              text: " here or contact us at ",
                                              style: new TextStyle(
                                                fontFamily: 'OpenSans',
                                                fontSize: 15.sp,
                                                fontWeight: FontWeight.normal,
                                                color: Colors.black,
                                              ),
                                            ),
                                            new TextSpan(
                                              text: 'privacy@shuftipro.com',
                                              style: new TextStyle(
                                                fontFamily: 'OpenSans',
                                                fontSize: 15.sp,
                                                fontWeight: FontWeight.normal,
                                                decoration:
                                                    TextDecoration.underline,
                                                color: Color(0xff2b6ad8),
                                              ),
                                              recognizer:
                                                  new TapGestureRecognizer()
                                                    ..onTap = () {
                                                      launch(
                                                          'mailto:privacy@shuftipro.com');
                                                    },
                                            ),
                                          ],
                                        ),
                                      ),
                                    )
                                  : Container()
                            ],
                          ),
                        ),
                      ),
                      Spacer(),
                      Padding(
                        padding: EdgeInsets.only(
                          left: 15.w,
                          right: 15.w,
                        ),
                        child: Container(
                          height: 45.h,
                          width: 1.sw,
                          child: RaisedButton(
                            onPressed: () {
                              {
                                verificationTypeList = [];
                                indexTempList = [];
                                isCheck = false;
                                Navigator.pushAndRemoveUntil(
                                    context,
                                    MaterialPageRoute(
                                        builder: (context) => userHome),
                                    (route) => false);
                                if (event == 'internet.connection') {
                                  widget
                                      .callback("internet.connection.problem");
                                } else {
                                  widget.callback(APIResponse);
                                }
                              }
                            },
                            textColor: Colors.white,
                            color: Color(0xff2b6ad8),
                            child: new Text(
                              "Proceed",
                              textAlign: TextAlign.center,
                              style: new TextStyle(
                                fontFamily: 'OpenSans',
                                fontSize: 17.sp,
                                fontWeight: FontWeight.bold,
                                color: Colors.white,
                              ),
                            ),
                          ),
                        ),
                      ),
                      Spacer(),
                      Padding(
                        padding: EdgeInsets.only(
                          bottom: 14.w,
                        ),
                        child: Container(
                            height: 13.h,
                            child: Image.asset("assets/sdk/footerImage.png", package: 'shufti_test')),
                      ),
                    ],
                  ),
                ));
  }
}

class DocType extends StatelessWidget {
  var typeList = verificationTypeList.asMap();
  final CalbackFunction callback;

  DocType({Key key, this.callback}) : super(key: key);

  AssetImage getIcon(key) {
    switch (key) {
      case 'utility_bill':
        return (AssetImage("assets/sdk/utility_bill_type.png", package: 'shufti_test'));
        break;
      case 'bank_statement':
        return (AssetImage("assets/sdk/bank_statement_type.png", package: 'shufti_test'));
        break;
      case 'printed':
        return (AssetImage("assets/sdk/printed_type.png", package: 'shufti_test'));
        break;
      case 'handwritten':
        return (AssetImage("assets/sdk/handwritten_type.png", package: 'shufti_test'));
        break;
      case 'passport':
        return (AssetImage("assets/sdk/passport.png", package: 'shufti_test'));
        break;
      case 'id_card':
        return (AssetImage("assets/sdk/id_card.png", package: 'shufti_test'));
        break;
      case 'credit_or_debit_card':
        return (AssetImage("assets/sdk/credit_card.png", package: 'shufti_test'));
        break;
      case 'driving_license':
        return (AssetImage("assets/sdk/driving_license.png", package: 'shufti_test'));
        break;

      default:
        {
          return (AssetImage("assets/sdk/document.png", package: 'shufti_test'));
        }
    }
  }

  @override
  Widget build(BuildContext context) {
    Future<bool> _onBackPressed() {
      Navigator.pop(context);
    }

    typeList = verificationTypeList.asMap();
    requestedPayloadTemp = requestedPayload[typeList[0]];
    if (requestedPayloadTemp.containsKey("supported_types") &&
        requestedPayloadTemp["supported_types"] != null &&
        requestedPayloadTemp["supported_types"] != "") {
      if (supportedTypes_list == null) {
        supportedTypes_list = requestedPayloadTemp["supported_types"];
      }
    } else {
      if (typeList[0] == "document" || typeList[0] == "document_two") {
        supportedTypes_list = [
          "passport",
          "id_card",
          "driving_license",
          "credit_or_debit_card",
        ];
      } else if (typeList[0] == "address") {
        supportedTypes_list = ["id_card", "utility_bill", "bank_statement"];
      } else if (typeList[0] == "consent") {
        supportedTypes_list = ["printed", "handwritten"];
      }
    }
    String verificationType = printVal(typeList[0]);

    // ScreenUtil.init(new BoxConstraints.loose(Size(360, 690)));
    return Scaffold(
      backgroundColor: Colors.white,
      appBar: AppBar(
        elevation: 0.0,
        centerTitle: true,
        title: Text(
          "$verificationType Verification",
          style: TextStyle(
            color: Colors.black,
            fontFamily: 'OpenSans',
            fontWeight: FontWeight.w500,
          ),
        ),
        backgroundColor: white,
        automaticallyImplyLeading: true,
        leading: GestureDetector(
          onTap: () {
            _onBackPressed();
          },
          child: Container(
            height: 27,
            width: 27,
            alignment: Alignment.centerLeft,
            margin: const EdgeInsets.fromLTRB(20, 0, 0, 0),
            child: Image(
              image: AssetImage("assets/sdk/back_icon.png", package: 'shufti_test'),
              height: 20,
              width: 20,
            ),
          ),
        ),
      ),
      body: Column(
        children: <Widget>[
          Padding(
            padding: EdgeInsets.only(
              top: 55.w,
              left: 25.w,
              bottom: 35.w,
              right: 25.w,
            ),
            child: Container(
              alignment: Alignment.topLeft,
              child: verificationType.toLowerCase() == "consent"
                  ? new Text(
                      "Choose the type of \nconsent note",
                      style: new TextStyle(
                        fontFamily: 'OpenSans',
                        fontSize: 20.sp,
                        fontWeight: FontWeight.bold,
                        color: Colors.black,
                      ),
                    )
                  : new Text(
                      "Select a document \nfor verification",
                      style: new TextStyle(
                        fontFamily: 'OpenSans',
                        fontSize: 20.sp,
                        fontWeight: FontWeight.bold,
                        color: Colors.black,
                      ),
                    ),
            ),
          ),
          for (var i in supportedTypes_list)
            Padding(
              padding: EdgeInsets.only(
                top: 15.w,
                left: 20.w,
                right: 20.w,
              ),
              child: GestureDetector(
                onTap: () {
                  requestedPayloadTemp = requestedPayload[typeList[0]];
                  requestedPayloadTemp["supported_types"] = [i];
                  requestedPayload[typeList[0]] = requestedPayloadTemp;
                  Navigator.push(
                    context,
                    MaterialPageRoute(
                        settings: RouteSettings(name: "/Ready"),
                        builder: (context) => Ready(
                              callback: (res) {
                                callback(res);
                                // print("\n\ncalling preview: " +
                                //     res.toString());
                              },
                            )),
                  );
                },
                child: Container(
                  height: 40.h,
                  decoration:
                      BoxDecoration(border: Border.all(color: Colors.black12)),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.start,
                    children: <Widget>[
                      Container(
                        margin: const EdgeInsets.fromLTRB(05, 0, 0, 0),
                        height: 20.h,
                        width: 0.07.sw,
                        child: Image(
                          image: getIcon(i),
                        ),
                      ),
                      Container(
                        margin: const EdgeInsets.fromLTRB(15, 0, 0, 0),
                        child: verificationType.toLowerCase() == "consent"
                            ? new Text(
                                printVal(i) + " Note",
                                style: new TextStyle(
                                    fontFamily: 'OpenSans',
                                    fontSize: 15.sp,
                                    color: Colors.black,
                                    fontWeight: FontWeight.w600),
                              )
                            : new Text(
                                printVal(i),
                                style: new TextStyle(
                                    fontFamily: 'OpenSans',
                                    fontSize: 15.sp,
                                    color: Colors.black,
                                    fontWeight: FontWeight.w600),
                              ),
                      ),
                      Spacer(),
                      Container(
                        height: 18.h,
                        width: 0.06.sw,
                        decoration: BoxDecoration(
                            border: Border.all(color: Colors.black12)),
                        alignment: Alignment.centerRight,
                        margin: const EdgeInsets.fromLTRB(0, 0, 10, 0),
                        child: Image(
                          image: AssetImage("assets/sdk/continue_next.png", package: 'shufti_test'),
                          height: 13.h,
                          width: 0.04.sw,
                        ),
                      ),
                    ],
                  ),
                ),
              ),
            ),
          Spacer(),
          Padding(
            padding: EdgeInsets.only(
              bottom: 14.w,
            ),
            child: Container(
                height: 13.h, child: Image.asset("assets/sdk/footerImage.png", package: 'shufti_test')),
          ),
        ],
      ),
    );
  }
}

class ImageCapture extends StatefulWidget {
  final CalbackFunction callback;

  const ImageCapture({Key key, this.callback}) : super(key: key);

  @override
  _ImageCaptureState createState() {
    return _ImageCaptureState();
  }
}

class _ImageCaptureState extends State<ImageCapture> {
  CameraController controller;
  var typeList = verificationTypeList.asMap();
  String videoPath;
  List<CameraDescription> cameras;
  int selectedCameraIdx;
  final GlobalKey<ScaffoldState> _scaffoldKey = GlobalKey<ScaffoldState>();
  @override
  bool isFlash = false;

  void flashCheck(bool newVal) => setState(() {
        if (selectedCameraIdx == 0) {
          isFlash = newVal;
          if (isFlash) {
            controller.setFlashMode(FlashMode.torch);
          } else {
            controller.setFlashMode(FlashMode.off);
          }
        }
      });

  void initState() {
    super.initState();

    availableCameras().then((availableCameras) {
      cameras = availableCameras;

      if (cameras.length > 0) {
        setState(() {
          if (typeList[0] == "face") {
            selectedCameraIdx = 1;
            cameraDirection = 1;
          } else {
            selectedCameraIdx = 0;
            cameraDirection = 0;
          }
        });

        _onCameraSwitched(cameras[selectedCameraIdx]).then((void v) {});
      }
    }).catchError((err) {});
  }

  @override
  Widget build(BuildContext context) {
    Future<bool> _onBackPressed() {
      // if(turn == 1){
      //   print("HELLO $turn");
      for (int i = 0; i < camCount; i++) {
        Navigator.pop(context);
      }
      camCount = 1;
      // }
      // else if(turn == 2){
      //   print("HELLO2 $turn");
      //   Navigator.pop(context);
      //   Navigator.pop(context);
      // }
      // else{
      //   print("HELLO3 $turn");
      //   Navigator.pop(context);
      //   Navigator.pop(context);
      //   Navigator.pop(context);
      // }
    }

    // ScreenUtil.init(new BoxConstraints.loose(Size(360, 690)));
    return WillPopScope(
        onWillPop: _onBackPressed,
        child: Scaffold(
          backgroundColor: Colors.white,
          key: _scaffoldKey,
          appBar: AppBar(
            elevation: 0.0,
            centerTitle: true,
            title: Text(
              "Take a photo",
              style: TextStyle(
                color: Colors.black,
                fontFamily: 'OpenSans',
                fontWeight: FontWeight.bold,
              ),
            ),
            backgroundColor: white,
            automaticallyImplyLeading: true,
            leading: GestureDetector(
              onTap: () {
                _onBackPressed();
              },
              child: Container(
                height: 27,
                width: 27,
                alignment: Alignment.centerLeft,
                margin: const EdgeInsets.fromLTRB(20, 0, 0, 0),
                child: Image(
                  image: AssetImage("assets/sdk/back_icon.png", package: 'shufti_test'),
                  height: 20,
                  width: 20,
                ),
              ),
            ),
          ),
          body: Center(
            child: Column(
              children: <Widget>[
                Expanded(flex: 7, child: _cameraPreviewWidget()),
                Expanded(
                  flex: 1,
                  child: Row(
                    children: [
                      Spacer(),
                      Container(
                          child: GestureDetector(
                        onTap: () {
                          flashCheck(!isFlash);
                        },
                        child: Image(
                          image: selectedCameraIdx == 0
                              ? isFlash
                                  ? AssetImage(
                                      "assets/sdk/ic_flash_on_white.png", package: 'shufti_test')
                                  : AssetImage(
                                      "assets/sdk/ic_flash_off_white.png", package: 'shufti_test')
                              : AssetImage("assets/sdk/ic_flash_off_white.png", package: 'shufti_test'),
                          height: 22,
                          width: 22,
                        ),
                      )),
                      Spacer(),
                      Container(
                        child: GestureDetector(
                          onTap: () async {
                            flashCheck(false);
                            try {
                              turn = 2;
                              final image = await controller.takePicture();
                              // controller.dispose();
                              // Navigator.pop(context);
                              camCount = camCount + 1;
                              Navigator.push(
                                context,
                                MaterialPageRoute(
                                  builder: (context) => DisplayPictureScreen(
                                    imagePath: image?.path,
                                    callback: (res) {
                                      widget.callback(res);
                                      // print("\n\ncalling preview: " +
                                      //     res.toString());
                                    },
                                  ),
                                ),
                              );
                            } catch (e) {
                              print(e);
                            }
                          },
                          child: Image(
                            image:
                                AssetImage("assets/sdk/button_red_capture.png", package: 'shufti_test'),
                            height: 55.h,
                            width: 0.3.sw,
                          ),
                        ),
                      ),
                      Spacer(),
                      Container(
                        child: GestureDetector(
                          onTap: () {
                            _onSwitchCamera();
                          },
                          child: Image(
                            image: AssetImage("assets/sdk/ic_camera_front.png", package: 'shufti_test'),
                            height: 22,
                            width: 22,
                          ),
                        ),
                      ),
                      Spacer(),
                    ],
                  ),
                )
              ],
            ),
          ),
        ));
  }

  Widget _cameraPreviewWidget() {
    if (controller == null || !controller.value.isInitialized) {
      return const Text(
        'Loading',
        style: TextStyle(
          fontFamily: 'OpenSans',
          color: Colors.white,
          fontSize: 20.0,
          fontWeight: FontWeight.w900,
        ),
      );
    }

    return AspectRatio(
      aspectRatio: controller.value.aspectRatio,
      child: CameraPreview(controller),
    );
  }

  Future<void> _onCameraSwitched(CameraDescription cameraDescription) async {
    // if (controller != null) {
    //   await controller.dispose();
    // }

    controller = CameraController(cameraDescription, ResolutionPreset.medium);

    controller.addListener(() {
      if (mounted) {
        setState(() {});
      }

      if (controller.value.hasError) {
        Scaffold.of(context).showSnackBar(SnackBar(
          content: Text('Camera error ${controller.value.errorDescription}'),
        ));
      }
    });

    try {
      await controller.initialize();
    } on CameraException catch (e) {
      _showCameraException(e);
    }

    if (mounted) {
      setState(() {});
    }
  }

  void _onSwitchCamera() {
    if (selectedCameraIdx == 0) {
      selectedCameraIdx = 1;
      cameraDirection = 1;
    } else if (selectedCameraIdx == 1) {
      selectedCameraIdx = 0;
      cameraDirection = 0;
      isFlash = false;
    }
    CameraDescription selectedCamera = cameras[selectedCameraIdx];

    _onCameraSwitched(selectedCamera);

    setState(() {
      selectedCameraIdx = selectedCameraIdx;
    });
  }

  void _showCameraException(CameraException e) {
    String errorText = 'Error: ${e.code}\nError Message: ${e.description}';
    print(errorText);
  }
}

class DisplayPictureScreen extends StatelessWidget {
  final String imagePath;
  final CalbackFunction callback;

  const DisplayPictureScreen({Key key, this.imagePath, this.callback})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    var typeList = verificationTypeList.asMap();
    void YesContinue() {
      turn = 1;
      File file = File(imagePath);
      Uint8List bytes = file.readAsBytesSync();
      String base64Image = base64Encode(bytes);

      typeList = verificationTypeList.asMap();
      if (typeList[0] == "face") {
        requestedPayloadTemp = requestedPayload["face"];
        requestedPayloadTemp["proof"] = base64Image;
        requestedPayload["face"] = requestedPayloadTemp;
      } else if (typeList[0] == "document") {
        if (!isBackside) {
          requestedPayloadTemp = requestedPayload["document"];
          requestedPayloadTemp["proof"] = base64Image;
          requestedPayload["document"] = requestedPayloadTemp;
        } else {
          requestedPayloadTemp = requestedPayload["document"];
          requestedPayloadTemp["additional_proof"] = base64Image;
          requestedPayload["document"] = requestedPayloadTemp;
        }
      } else if (typeList[0] == "document_two") {
        if (!isBackside) {
          requestedPayloadTemp = requestedPayload["document_two"];
          requestedPayloadTemp["proof"] = base64Image;
          requestedPayload["document_two"] = requestedPayloadTemp;
        } else {
          requestedPayloadTemp = requestedPayload["document_two"];
          requestedPayloadTemp["additional_proof"] = base64Image;
          requestedPayload["document_two"] = requestedPayloadTemp;
        }
      } else if (typeList[0] == "address") {
        // if (!isBackside) {
        requestedPayloadTemp = requestedPayload["address"];
        requestedPayloadTemp["proof"] = base64Image;
        requestedPayload["address"] = requestedPayloadTemp;
        // } else {
        //   requestedPayloadTemp = requestedPayload["address"];
        //   requestedPayloadTemp["additional_proof"] = base64Image;
        //   requestedPayload["address"] = requestedPayloadTemp;
        // }
      } else if (typeList[0] == "consent") {
        requestedPayloadTemp = requestedPayload["consent"];
        requestedPayloadTemp["proof"] = base64Image;
        requestedPayload["consent"] = requestedPayloadTemp;
      }

      requestedPayloadTemp = requestedPayload[typeList[0]];

      if (requestedPayloadTemp["supported_types"].toString().toLowerCase() ==
              "[passport]" &&
          !isBackside &&
          (typeList[0] == "document" || typeList[0] == "document_two")) {
        if (requestedPayloadTemp["backside_proof_required"].toString() ==
                null ||
            requestedPayloadTemp["backside_proof_required"].toString() == "" ||
            requestedPayloadTemp["backside_proof_required"].toString() == "0") {
          verificationTypeList.removeAt(0);
          supportedTypes_list = null;
          typeList = verificationTypeList.asMap();
          isBackside = false;
          isCheck = false;
          if (verificationTypeList.length == 0) {
            if (asyncSDK &&
                !requestedPayload.containsKey('background_checks') &&
                !requestedPayload.containsKey('phone')) {
              check().then((intenet) {
                if (intenet != null && intenet) {
                  ApiMethods().apiRequest().then((response) {
                    callback(response);
                  });
                } else {
                  callback("internet.connection.problem");
                }
                Navigator.pushAndRemoveUntil(
                    context,
                    MaterialPageRoute(builder: (context) => userHome),
                    (route) => false);
                callback("event: request.pending, error: '', reference: " +
                    createdPayloadSDK["reference"]);
                // callback(APIResponse);
              });
            } else {
              Navigator.push(
                context,
                MaterialPageRoute(
                    builder: (context) => Result(
                          callback: (res) {
                            callback(res);
                            // print("\n\ncalling preview: " +
                            //     res.toString());
                          },
                        )),
              );
            }
          } else {
            Navigator.push(
              context,
              MaterialPageRoute(
                  builder: (context) => ShowInstructions(
                        callback: (res) {
                          callback(res);
                          // print("\n\ncalling preview: " +
                          //     res.toString());
                        },
                      )),
            );
          }
        } else if (requestedPayloadTemp["backside_proof_required"].toString() ==
            "1") {
          isBackside = true;
          Navigator.push(
            context,
            MaterialPageRoute(
                builder: (context) => Ready(
                      callback: (res) {
                        callback(res);
                        // print("\n\ncalling preview: " +
                        //     res.toString());
                      },
                    )),
          );
        }
      } else if (!isBackside &&
          (typeList[0] == "document" || typeList[0] == "document_two")) {
        isBackside = true;
        Navigator.push(
          context,
          MaterialPageRoute(
              builder: (context) => Ready(
                    callback: (res) {
                      callback(res);
                      // print("\n\ncalling preview: " +
                      //     res.toString());
                    },
                  )),
        );
      } else {
        verificationTypeList.removeAt(0);
        supportedTypes_list = null;
        typeList = verificationTypeList.asMap();
        isBackside = false;
        isCheck = false;
        if (verificationTypeList.length == 0) {
          if (asyncSDK &&
              !requestedPayload.containsKey('background_checks') &&
              !requestedPayload.containsKey('phone')) {
            check().then((intenet) {
              if (intenet != null && intenet) {
                ApiMethods().apiRequest().then((response) {
                  callback(response);
                });
              } else {
                callback("internet.connection.problem");
              }
              Navigator.pushAndRemoveUntil(
                  context,
                  MaterialPageRoute(builder: (context) => userHome),
                  (route) => false);
              callback("event: request.pending, error: '', reference: " +
                  createdPayloadSDK["reference"]);
              // callback(APIResponse);
            });
          } else {
            Navigator.push(
              context,
              MaterialPageRoute(
                  builder: (context) => Result(
                        callback: (res) {
                          callback(res);
                          // print("\n\ncalling preview: " +
                          //     res.toString());
                        },
                      )),
            );
          }
        } else {
          Navigator.push(
            context,
            MaterialPageRoute(
                builder: (context) => ShowInstructions(
                      callback: (res) {
                        callback(res);
                        // print("\n\ncalling preview: " +
                        //     res.toString());
                      },
                    )),
          );
        }
      }
    }

    Future<bool> _onBackPressed() {
      for (int i = 0; i < camCount; i++) {
        Navigator.pop(context);
      }
      camCount = 1;
      // Navigator.of(context).pop();
      // Navigator.push(
      //   context,
      //   MaterialPageRoute(builder: (context) => ImageCapture(callback: (res) {
      //     callback(res);
      //     // print("\n\ncalling preview: " +
      //     //     res.toString());
      //   },)),
      // );
    }

    // ScreenUtil.init(new BoxConstraints.loose(Size(360, 690)));
    return WillPopScope(
        onWillPop: _onBackPressed,
        child: Scaffold(
          backgroundColor: Colors.white,
          appBar: AppBar(
            backgroundColor: white,
            elevation: 0.0,
            centerTitle: true,
            title: Padding(
              padding: const EdgeInsets.only(
                top: 15,
              ),
              child: Text(
                " Confirmation",
                style: TextStyle(
                  color: Colors.black,
                  fontFamily: 'OpenSans',
                  fontWeight: FontWeight.bold,
                  fontSize: 20.sp,
                ),
              ),
            ),
            automaticallyImplyLeading: true,
            leading: GestureDetector(
              onTap: () {
                _onBackPressed();
              },
              child: Container(
                height: 27,
                width: 27,
                alignment: Alignment.centerLeft,
                margin: const EdgeInsets.fromLTRB(20, 0, 0, 0),
                child: Image(
                  image: AssetImage("assets/sdk/back_icon.png", package: 'shufti_test'),
                  height: 20,
                  width: 20,
                ),
              ),
            ),
          ),
          body: Column(
            children: <Widget>[
              Expanded(
                  flex: 1,
                  child: Container(
                    height: 50.h,
                    color: Colors.white,
                    child: Column(
                      children: [
                        Spacer(),
                        typeList[0] == 'face'
                            ? Padding(
                                padding: EdgeInsets.only(
                                  left: 15.w,
                                  right: 15.w,
                                ),
                                child: Container(
                                  alignment: Alignment.topLeft,
                                  child: Text(
                                    "Can you see your full face in the photo?",
                                    textAlign: TextAlign.start,
                                    style: new TextStyle(
                                      fontFamily: 'OpenSans',
                                      fontSize: 16.sp,
                                      fontWeight: FontWeight.w600,
                                      color: Colors.black,
                                    ),
                                  ),
                                ),
                              )
                            : typeList[0] == 'document'
                                ? Padding(
                                    padding: EdgeInsets.only(
                                      left: 15.w,
                                      right: 15.w,
                                    ),
                                    child: Container(
                                      alignment: Alignment.topLeft,
                                      child: Text(
                                        "Can you see the entire document clearly in the photo?",
                                        textAlign: TextAlign.start,
                                        style: new TextStyle(
                                          fontFamily: 'OpenSans',
                                          fontSize: 16.sp,
                                          fontWeight: FontWeight.w600,
                                          color: Colors.black,
                                        ),
                                      ),
                                    ),
                                  )
                                : typeList[0] == 'document_two'
                                    ? Padding(
                                        padding: EdgeInsets.only(
                                          left: 15.w,
                                          right: 15.w,
                                        ),
                                        child: Container(
                                          alignment: Alignment.topLeft,
                                          child: Text(
                                            "Can you see the entire document clearly in the photo?",
                                            textAlign: TextAlign.start,
                                            style: new TextStyle(
                                              fontFamily: 'OpenSans',
                                              fontSize: 16.sp,
                                              fontWeight: FontWeight.w600,
                                              color: Colors.black,
                                            ),
                                          ),
                                        ),
                                      )
                                    : typeList[0] == 'address'
                                        ? Padding(
                                            padding: EdgeInsets.only(
                                              left: 15.w,
                                              right: 15.w,
                                            ),
                                            child: Container(
                                              alignment: Alignment.topLeft,
                                              child: Text(
                                                "Can you see the entire address document clearly in the photo?",
                                                textAlign: TextAlign.start,
                                                style: new TextStyle(
                                                  fontFamily: 'OpenSans',
                                                  fontSize: 16.sp,
                                                  fontWeight: FontWeight.w600,
                                                  color: Colors.black,
                                                ),
                                              ),
                                            ),
                                          )
                                        : Padding(
                                            padding: EdgeInsets.only(
                                              left: 15.w,
                                              right: 15.w,
                                            ),
                                            child: Container(
                                              alignment: Alignment.topLeft,
                                              child: Text(
                                                "Can you see your entire face and consent note clearly in the photo?",
                                                textAlign: TextAlign.start,
                                                style: new TextStyle(
                                                  fontFamily: 'OpenSans',
                                                  fontSize: 16.sp,
                                                  fontWeight: FontWeight.w600,
                                                  color: Colors.black,
                                                ),
                                              ),
                                            ),
                                          ),
                        Spacer(),
                      ],
                    ),
                  )),
              Expanded(
                flex: 8,
                child: Padding(
                  padding: EdgeInsets.only(
                    left: 10.w,
                    right: 15.w,
                  ),
                  child: cameraDirection == 1
                      ? Transform(
                          alignment: Alignment.center,
                          child: Image.file(File(imagePath)),
                          transform: Matrix4.rotationY(Math.pi))
                      : Image.file(File(imagePath)),
                ),
              ),
              Expanded(
                flex: 2,
                child: Column(
                  children: <Widget>[
                    Expanded(
                      flex: 9,
                      child: Row(
                        children: [
                          Expanded(
                            flex: 1,
                            child: Padding(
                              padding: EdgeInsets.only(
                                left: 15.w,
                                right: 5.w,
                              ),
                              child: Container(
                                height: 45.h,
                                child: OutlinedButton(
                                  child: Text(
                                    "Retake",
                                    style: TextStyle(
                                        fontFamily: 'OpenSans',
                                        fontSize: 18.sp,
                                        fontWeight: FontWeight.bold,
                                        color: Colors.black),
                                  ),
                                  style: OutlinedButton.styleFrom(
                                    shape: RoundedRectangleBorder(
                                        borderRadius: BorderRadius.circular(5)),
                                  ),
                                  onPressed: () {
                                    turn = 3;
                                    camCount = camCount + 1;
                                    // Navigator.pop(context);
                                    Navigator.push(
                                      context,
                                      MaterialPageRoute(
                                          builder: (context) => ImageCapture(
                                                callback: (res) {
                                                  callback(res);
                                                  // print("\n\ncalling preview: " +
                                                  //     res.toString());
                                                },
                                              )),
                                    );
                                  },
                                ),
                              ),
                            ),
                          ),
                          Expanded(
                            flex: 1,
                            child: Padding(
                              padding: EdgeInsets.only(
                                left: 0.w,
                                right: 15.w,
                              ),
                              child: Container(
                                height: 45.h,
                                child: OutlinedButton(
                                  style: OutlinedButton.styleFrom(
                                    shape: RoundedRectangleBorder(
                                        borderRadius: BorderRadius.circular(5)),
                                    backgroundColor: Color(0xff2b6ad8),
                                  ),
                                  child: Text(
                                    'Yes, Continue',
                                    style: TextStyle(
                                      fontFamily: 'OpenSans',
                                      fontSize: 18.sp,
                                      fontWeight: FontWeight.bold,
                                      color: Colors.white,
                                    ),
                                  ),
                                  onPressed: () => YesContinue(),
                                ),
                              ),
                            ),
                          )
                        ],
                      ),
                    ),
                    Expanded(
                      flex: 2,
                      child: Padding(
                        padding: EdgeInsets.only(
                          bottom: 14.w,
                        ),
                        child: Container(
                            height: 13.h,
                            child: Image.asset("assets/sdk/footerImage.png", package: 'shufti_test')),
                      ),
                    ),
                  ],
                ),
              )
            ],
          ),
        ));
  }
}

class VideoRecorder extends StatefulWidget {
  final CalbackFunction callback;

  const VideoRecorder({Key key, this.callback}) : super(key: key);

  @override
  _VideoRecorderState createState() {
    return _VideoRecorderState();
  }
}

class _VideoRecorderState extends State<VideoRecorder> {
  var typeList = verificationTypeList.asMap();

  void startTimer() {
    countDownTimer = new CountdownTimer(
      new Duration(seconds: _start),
      new Duration(seconds: 1),
    );

    var sub = countDownTimer.listen(null);
    sub.onData((duration) {
      setState(() {
        _current = _start - duration.elapsed.inSeconds;
      });
    });

    sub.onDone(() {
      onStopButtonPressed();
      sub.cancel();
    });
  }

  CameraController controller;
  String videoPath;

  List<CameraDescription> cameras;
  int selectedCameraIdx;
  final GlobalKey<ScaffoldState> _scaffoldKey = GlobalKey<ScaffoldState>();
  bool isFlash = false;
  bool cameraSwitch = true;
  var appBarTitleText = "Record a video";
  var isBackBtn = true;

  @override
  void flashCheck(bool newVal) => setState(() {
        isFlash = newVal;
        if (isFlash) {
          controller.setFlashMode(FlashMode.torch);
        } else {
          controller.setFlashMode(FlashMode.off);
        }
      });

  void initState() {
    super.initState();

    availableCameras().then((availableCameras) {
      cameras = availableCameras;

      if (cameras.length > 0) {
        setState(() {
          if (typeList[0] == "face") {
            selectedCameraIdx = 1;
            cameraDirection = 1;
          } else {
            selectedCameraIdx = 0;
            cameraDirection = 0;
          }
        });

        _onCameraSwitched(cameras[selectedCameraIdx]).then((void v) {});
      }
    }).catchError((err) {
      print('Error: $err.code\nError Message: $err.message');
    });
  }

  @override
  Widget build(BuildContext context) {
    Future<bool> _onBackPressed() {
      // if(turn == 1){
      //   Navigator.pop(context);
      // }
      // else{
      //   Navigator.pop(context);
      //   Navigator.pop(context);
      // }
      // if(turn == 1){
      //   print("HELLO $turn");
      for (int i = 0; i < camCount; i++) {
        Navigator.pop(context);
      }
      camCount = 1;
      // }
      // else if(turn == 2){
      //   print("HELLO2 $turn");
      //   Navigator.pop(context);
      //   Navigator.pop(context);
      // }
      // else{
      //   print("HELLO3 $turn");
      //   Navigator.pop(context);
      //   Navigator.pop(context);
      //   Navigator.pop(context);
      // }
    }

    // ScreenUtil.init(new BoxConstraints.loose(Size(360, 690)));
    return WillPopScope(
        onWillPop: _onBackPressed,
        child: Scaffold(
          backgroundColor: Colors.white,
          key: _scaffoldKey,



          appBar:
          controller != null && controller.value.isInitialized && controller.value.isRecordingVideo
              ? AppBar(
            elevation: 0.0,
            centerTitle: true,
            title: Row(
                children: <Widget>[
                  Text(
                    "Recording",
                    style: TextStyle(
                      color: Colors.black,
                      fontFamily: 'OpenSans',
                      fontWeight: FontWeight.bold,
                    ),),
                  Spacer(),
                  _current==10 ?
                  Text("00:$_current",
                    style: TextStyle(
                      fontSize: 15.0,
                      color: Color(0xff2b6ad8),
                      fontFamily: 'OpenSans',
                      fontWeight: FontWeight.bold,
                    ),) : Text("00:0$_current",
                    style: TextStyle(
                      fontSize: 15.0,
                      color: Color(0xff2b6ad8),
                      fontFamily: 'OpenSans',
                      fontWeight: FontWeight.bold,
                    ),)
                ]
            ),

            backgroundColor: white,
            automaticallyImplyLeading: isBackBtn,
          )
              : AppBar(
            centerTitle: true,
            elevation: 0.0,
            title: const Text(
              "Record a video",
              style: TextStyle(
                color: Colors.black,
                fontFamily: 'OpenSans',
                fontWeight: FontWeight.bold,
              ),),
            backgroundColor: white,
            leading: GestureDetector(
              onTap: () {
                _onBackPressed();
              },
              child: Container(
                height: 27,
                width: 27,
                alignment: Alignment.centerLeft,
                margin: const EdgeInsets.fromLTRB(20, 0, 0, 0),
                child: Image(
                  image: AssetImage("assets/sdk/back_icon.png", package: 'shufti_test'),
                  height: 20,
                  width: 20,
                ),
              ),
            ),
          ),

          // appBar: AppBar(
          //   elevation: 0.0,
          //   centerTitle: true,
          //   title: Row(
          //       children: <Widget>[
          //         Text(
          //           "Record a video",
          //           style: TextStyle(
          //             color: Colors.black,
          //             fontFamily: 'OpenSans',
          //             fontWeight: FontWeight.bold,
          //           ),),
          //       ]
          //   ),
          //   backgroundColor: white,
          // ),
          body: Center(
            child: Column(
              children: <Widget>[
                Expanded(flex: 7, child: _cameraPreviewWidget()),
                Expanded(
                  flex: 1,
                  child: Row(
                    children: [
                      Spacer(),
                      Container(
                          child: GestureDetector(
                            onTap: () {
                              flashCheck(!isFlash);
                            },
                            child: Image(
                              image: selectedCameraIdx == 0
                                  ? isFlash
                                  ? AssetImage("assets/sdk/ic_flash_on_white.png", package: 'shufti_test')
                                  : AssetImage(
                                  "assets/sdk/ic_flash_off_white.png", package: 'shufti_test')
                                  : AssetImage("assets/sdk/ic_flash_off_white.png", package: 'shufti_test'),
                              height: 22,
                              width: 22,
                            ),
                          )),
                      Spacer(),
                      Container(
                        child: GestureDetector(
                          onTap: () async {
                            try {
                              controller != null && controller.value.isInitialized
                                  ? !controller.value.isRecordingVideo
                              // ? _startVideoRecording()
                                  ? onVideoRecordButtonPressed()
                                  : Future.delayed(
                                  const Duration(milliseconds: 1000), () {
                                controller != null && controller.value.isInitialized && controller.value.isRecordingVideo ?
                                onStopButtonPressed() : null;
                                // _stopVideoRecording() : null;
                              })
                                  : null;
                            } catch (e) {
                              print(e);
                            }
                          },
                          child: Image(
                            image:
                            AssetImage("assets/sdk/button_red_capture.png", package: 'shufti_test'),
                            height: 55.h,
                            width: 0.3.sw,
                          ),
                        ),
                      ),
                      Spacer(),
                      Container(
                        child: GestureDetector(
                          onTap: () {
                            cameraSwitch && controller != null && controller.value.isInitialized && !controller.value.isRecordingVideo
                                ? _onSwitchCamera() : null;
                          },
                          child: Image(
                            image: AssetImage("assets/sdk/ic_camera_front.png", package: 'shufti_test'),
                            height: 22,
                            width: 22,
                          ),
                        ),
                      ),
                      Spacer(),
                    ],
                  ),
                )
              ],
            ),
          ),
        ));
  }

  Widget _cameraPreviewWidget() {
    if (controller == null || !controller.value.isInitialized) {
      return const Text(
        'Loading',
        style: TextStyle(
          fontFamily: 'OpenSans',
          color: Colors.white,
          fontSize: 20.0,
          fontWeight: FontWeight.w900,
        ),
      );
    }

    return AspectRatio(
      aspectRatio: controller.value.aspectRatio,
      child: CameraPreview(controller),
    );
  }



  // @override
  // Widget build(BuildContext context) {
  //   Future<bool> _onBackPressed() {
  //     // if(turn == 1){
  //     //   Navigator.pop(context);
  //     // }
  //     // else{
  //     //   Navigator.pop(context);
  //     //   Navigator.pop(context);
  //     // }
  //     // if(turn == 1){
  //     //   print("HELLO $turn");
  //     for (int i = 0; i < camCount; i++) {
  //       Navigator.pop(context);
  //     }
  //     camCount = 1;
  //     // }
  //     // else if(turn == 2){
  //     //   print("HELLO2 $turn");
  //     //   Navigator.pop(context);
  //     //   Navigator.pop(context);
  //     // }
  //     // else{
  //     //   print("HELLO3 $turn");
  //     //   Navigator.pop(context);
  //     //   Navigator.pop(context);
  //     //   Navigator.pop(context);
  //     // }
  //   }
  //
  //   return Scaffold(
  //     key: _scaffoldKey,
  //     appBar: AppBar(
  //       elevation: 0.0,
  //       centerTitle: true,
  //       title: Row(
  //           children: <Widget>[
  //             Spacer(),
  //             Text(
  //               appBarTitleText,
  //               style: TextStyle(
  //                 color: Colors.black,
  //                 fontFamily: 'OpenSans',
  //                 fontWeight: FontWeight.bold,
  //             ),),
  //             Spacer(),
  //             _current==10 ?
  //             Text("00:$_current",
  //               style: TextStyle(
  //                 fontSize: 15.0,
  //               color: Color(0xff2b6ad8),
  //               fontFamily: 'OpenSans',
  //                 fontWeight: FontWeight.bold,
  //             ),) : Text("00:0$_current",
  //               style: TextStyle(
  //                 fontSize: 15.0,
  //                 color: Color(0xff2b6ad8),
  //                 fontFamily: 'OpenSans',
  //                 fontWeight: FontWeight.bold,
  //               ),)
  //           ]
  //       ),
  //
  //       backgroundColor: white,
  //       automaticallyImplyLeading: isBackBtn,
  //       leading: GestureDetector(
  //         onTap: () {
  //           _onBackPressed();
  //         },
  //         child: Container(
  //           height: 27,
  //           width: 27,
  //           alignment: Alignment.centerLeft,
  //           margin: const EdgeInsets.fromLTRB(20, 0, 0, 0),
  //           child: Image(
  //             image: AssetImage("assets/sdk/back_icon.png"),
  //             height: 20,
  //             width: 20,
  //           ),
  //         ),
  //       ),
  //     ),
  //
  //     body: Column(
  //       children: <Widget>[
  //         Expanded(
  //           child: Container(
  //             child: Padding(
  //               padding: const EdgeInsets.all(1.0),
  //               child: Center(
  //                 child: _cameraPreviewWidget(),
  //               ),
  //             ),
  //           ),
  //         ),
  //         Positioned(
  //                 left: 0,
  //                 right: 0,
  //                 bottom: 0,
  //                 child: Padding(
  //                   padding: const EdgeInsets.all(0.0),
  //                   child: Container(
  //                     color: white,
  //                     child: Padding(
  //                       padding: const EdgeInsets.all(5.0),
  //                       child: Row(
  //                         children: [
  //                           Spacer(),
  //                           Container(
  //                               child: GestureDetector(
  //                             onTap: () {
  //                               flashCheck(!isFlash);
  //                             },
  //                             child: Image(
  //                               image: selectedCameraIdx == 0
  //                                   ? isFlash
  //                                       ? AssetImage("assets/sdk/ic_flash_on_white.png")
  //                                       : AssetImage(
  //                                           "assets/sdk/ic_flash_off_white.png")
  //                                   : AssetImage("assets/sdk/ic_flash_off_white.png"),
  //                               height: 22,
  //                               width: 22,
  //                             ),
  //                           )),
  //                           Spacer(),
  //                           Container(
  //                             child: GestureDetector(
  //                               onTap: () async {
  //                                 try {
  //                                   controller != null && controller.value.isInitialized
  //                                       ? !controller.value.isRecordingVideo
  //                                           // ? _startVideoRecording()
  //                                           ? onVideoRecordButtonPressed()
  //                                           : Future.delayed(
  //                                               const Duration(milliseconds: 1000), () {
  //                                     controller != null && controller.value.isInitialized && controller.value.isRecordingVideo ?
  //                                               onStopButtonPressed() : null;
  //                                               // _stopVideoRecording() : null;
  //                                             })
  //                                       : null;
  //                                 } catch (e) {
  //                                   print(e);
  //                                 }
  //                               },
  //                               child: Image(
  //                                 image:
  //                                     AssetImage("assets/sdk/button_red_capture.png"),
  //                                 height: 55.h,
  //                                 width: 0.3.sw,
  //                               ),
  //                             ),
  //                           ),
  //                           Spacer(),
  //                           Container(
  //                             child: GestureDetector(
  //                               onTap: () {
  //                                 cameraSwitch && controller != null && controller.value.isInitialized && !controller.value.isRecordingVideo
  //                                     ? _onSwitchCamera() : null;
  //                               },
  //                               child: Image(
  //                                 image: AssetImage("assets/sdk/ic_camera_front.png"),
  //                                 height: 22,
  //                                 width: 22,
  //                               ),
  //                             ),
  //                           ),
  //                           Spacer(),
  //                         ],
  //                       ),
  //                     ),
  //                   ),
  //                 ),
  //               ),
  //         ],
  //     ),
  //   );
  // }
  //
  // Widget _cameraPreviewWidget() {
  //   final CameraController cameraController = controller;
  //
  //   if (cameraController == null || !cameraController.value.isInitialized) {
  //     return const Text(
  //       'Loading...',
  //       style: TextStyle(
  //         color: Colors.white,
  //         fontSize: 24.0,
  //         fontWeight: FontWeight.w900,
  //       ),
  //     );
  //   } else {
  //     return Listener(
  //       onPointerDown: (_) => _pointers++,
  //       onPointerUp: (_) => _pointers--,
  //       child: CameraPreview(
  //         controller,
  //         child: LayoutBuilder(
  //             builder: (BuildContext context, BoxConstraints constraints) {
  //               return GestureDetector(
  //                 behavior: HitTestBehavior.opaque,
  //                 onScaleStart: _handleScaleStart,
  //                 onScaleUpdate: _handleScaleUpdate,
  //                 onTapDown: (details) => onViewFinderTap(details, constraints),
  //               );
  //             }),
  //       ),
  //     );
  //   }
  // }

  void _handleScaleStart(ScaleStartDetails details) {
    _baseScale = _currentScale;
  }

  Future<void> _handleScaleUpdate(ScaleUpdateDetails details) async {
    // When there are not exactly two fingers on screen don't scale
    if (controller == null || _pointers != 2) {
      return;
    }

    _currentScale = (_baseScale * details.scale)
        .clamp(_minAvailableZoom, _maxAvailableZoom);

    await controller.setZoomLevel(_currentScale);
  }

  void onViewFinderTap(TapDownDetails details, BoxConstraints constraints) {
    if (controller == null) {
      return;
    }

    final CameraController cameraController = controller;

    final offset = Offset(
      details.localPosition.dx / constraints.maxWidth,
      details.localPosition.dy / constraints.maxHeight,
    );
    cameraController.setExposurePoint(offset);
    cameraController.setFocusPoint(offset);
  }

  Future<void> _onCameraSwitched(CameraDescription cameraDescription) async {
    // try {
    //   if (controller != null) {
    //     await controller.dispose();
    //   }
    // }catch(e){
    //
    // }
    controller = CameraController(cameraDescription, ResolutionPreset.high);
    controller.addListener(() {
      if (mounted) {
        setState(() {});
      }

      if (controller.value.hasError) {}
    });

    try {
      await controller.initialize();
    } on CameraException catch (e) {
      _showCameraException(e, "switched");
    }

    if (mounted) {
      setState(() {});
    }
  }

  void _onSwitchCamera() {
    if (selectedCameraIdx == 0) {
      selectedCameraIdx = 1;
      cameraDirection = 1;
    } else if (selectedCameraIdx == 1) {
      selectedCameraIdx = 0;
      cameraDirection = 0;
      isFlash = false;
    }

    CameraDescription selectedCamera = cameras[selectedCameraIdx];

    _onCameraSwitched(selectedCamera);

    setState(() {
      selectedCameraIdx = selectedCameraIdx;
    });
  }

  void onVideoRecordButtonPressed() {
    startVideoRecording().then((_) {
      if (mounted) setState(() {});
    });
  }

  void onStopButtonPressed() {
    stopVideoRecording().then((file) {
      if (mounted) setState(() {});
      if (file != null) {
        // videoFile = file;
        // _startVideoPlayer();
        // Image.file(File(file.path));
        controller.setFlashMode(FlashMode.off);
        // controller.dispose();
            videoPathToPass = file.path;
            camCount = camCount + 1;
            Navigator.push(
              context,
              MaterialPageRoute(
                  builder: (context) => VideoPlayerScreen(
                        callback: (res) {
                          widget.callback(res);
                        },
                      )),
            );

      }
    });
  }

  Future<void> startVideoRecording() async {
    final CameraController cameraController = controller;

    if (cameraController == null || !cameraController.value.isInitialized) {
      return;
    }

    if (cameraController.value.isRecordingVideo) {
      // A recording is already started, do nothing.
      return;
    }

    try {
      await cameraController.startVideoRecording();
      startTimer();
      setState(() {
        appBarTitleText = "Recording";
        isBackBtn = false;
      });
    } on CameraException catch (e) {
      _showCameraException(e, "start");
      return;
    }
  }

  Future<XFile> stopVideoRecording() async {
    final CameraController cameraController = controller;

    if (cameraController == null || !cameraController.value.isRecordingVideo) {
      return null;
    }

    try {
      countDownTimer.cancel();
      return cameraController.stopVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e, "Stop");
      return null;
    }
  }

  void _showCameraException(CameraException e, String s) {
    String errorText = s + ' Error: ${e.code}\nError Message: ${e.description}';
    print(errorText);
  }
}

class VideoPlayerScreen extends StatefulWidget {
  final CalbackFunction callback;

  VideoPlayerScreen({Key key, this.callback}) : super(key: key);

  @override
  _VideoPlayerScreenState createState() => _VideoPlayerScreenState(callback);
}

class _VideoPlayerScreenState extends State<VideoPlayerScreen> {
  VideoPlayerController _controller = VideoPlayerController.file(File(videoPathToPass));
  Future<void> _initializeVideoPlayerFuture;
  var typeList = verificationTypeList.asMap();
  final CalbackFunction callback;

  _VideoPlayerScreenState(this.callback);

  @override
  void initState() {
    // _controller = VideoPlayerController.network(
    //   // 'https://flutter.github.io/assets-for-api-docs/assets/videos/butterfly.mp4',
    //   videoPathToPass,
    // );
    _initializeVideoPlayerFuture = _controller.initialize();
    _controller.setLooping(true);
    _controller.setVolume(0.0);
    _controller.play();

    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();

    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    print("heko " + videoPathToPass);

    void YesContinue() {
      turn = 1;
      File file = File(videoPathToPass);
      Uint8List bytes = file.readAsBytesSync();
      String base64Image = base64Encode(bytes);

      typeList = verificationTypeList.asMap();
      if (typeList[0] == "face") {
        requestedPayloadTemp = requestedPayload["face"];
        requestedPayloadTemp["proof"] = base64Image;
        requestedPayload["face"] = requestedPayloadTemp;
      } else if (typeList[0] == "document") {
        if (!isBackside) {
          requestedPayloadTemp = requestedPayload["document"];
          requestedPayloadTemp["proof"] = base64Image;
          requestedPayload["document"] = requestedPayloadTemp;
        } else {
          requestedPayloadTemp = requestedPayload["document"];
          requestedPayloadTemp["additional_proof"] = base64Image;
          requestedPayload["document"] = requestedPayloadTemp;
        }
      } else if (typeList[0] == "document_two") {
        if (!isBackside) {
          requestedPayloadTemp = requestedPayload["document_two"];
          requestedPayloadTemp["proof"] = base64Image;
          requestedPayload["document_two"] = requestedPayloadTemp;
        } else {
          requestedPayloadTemp = requestedPayload["document_two"];
          requestedPayloadTemp["additional_proof"] = base64Image;
          requestedPayload["document_two"] = requestedPayloadTemp;
        }
      } else if (typeList[0] == "address") {
        // if (!isBackside) {
        requestedPayloadTemp = requestedPayload["address"];
        requestedPayloadTemp["proof"] = base64Image;
        requestedPayload["address"] = requestedPayloadTemp;
        // } else {
        //   requestedPayloadTemp = requestedPayload["address"];
        //   requestedPayloadTemp["additional_proof"] = base64Image;
        //   requestedPayload["address"] = requestedPayloadTemp;
        // }
      } else if (typeList[0] == "consent") {
        requestedPayloadTemp = requestedPayload["consent"];
        requestedPayloadTemp["proof"] = base64Image;
        requestedPayload["consent"] = requestedPayloadTemp;
      }

      requestedPayloadTemp = requestedPayload[typeList[0]];

      if (requestedPayloadTemp["supported_types"].toString().toLowerCase() ==
          "passport") {}

      if (requestedPayloadTemp["supported_types"].toString().toLowerCase() ==
          "[passport]" &&
          !isBackside &&
          (typeList[0] == "document" || typeList[0] == "document_two")) {
        if (requestedPayloadTemp["backside_proof_required"].toString() ==
            null ||
            requestedPayloadTemp["backside_proof_required"].toString() == "" ||
            requestedPayloadTemp["backside_proof_required"].toString() == "0") {
          verificationTypeList.removeAt(0);
          supportedTypes_list = null;
          typeList = verificationTypeList.asMap();
          isBackside = false;
          isCheck = false;
          if (verificationTypeList.length == 0) {
            if (asyncSDK &&
                !requestedPayload.containsKey('background_checks') &&
                !requestedPayload.containsKey('phone')) {
              check().then((intenet) {
                if (intenet != null && intenet) {
                  ApiMethods().apiRequest().then((response) {
                    callback(response);
                  });
                } else {
                  callback("internet.connection.problem");
                }
                Navigator.pushAndRemoveUntil(
                    context,
                    MaterialPageRoute(builder: (context) => userHome),
                        (route) => false);
                callback("event: request.pending, error: '', reference: " +
                    createdPayloadSDK["reference"]);
                // callback(APIResponse);
              });
            } else {
              Navigator.push(
                context,
                MaterialPageRoute(
                    builder: (context) => Result(
                      callback: (res) {
                        callback(res);
                        // print("\n\ncalling preview: " +
                        //     res.toString());
                      },
                    )),
              );
            }
          } else {
            Navigator.push(
              context,
              MaterialPageRoute(
                  builder: (context) => ShowInstructions(
                    callback: (res) {
                      callback(res);
                      // print("\n\ncalling preview: " +
                      //     res.toString());
                    },
                  )),
            );
          }
        } else if (requestedPayloadTemp["backside_proof_required"].toString() ==
            "1") {
          isBackside = true;
          Navigator.push(
            context,
            MaterialPageRoute(
                builder: (context) => Ready(
                  callback: (res) {
                    callback(res);
                    // print("\n\ncalling preview: " +
                    //     res.toString());
                  },
                )),
          );
        }
      } else if (!isBackside &&
          (typeList[0] == "document" || typeList[0] == "document_two")) {
        isBackside = true;
        Navigator.push(
          context,
          MaterialPageRoute(
              builder: (context) => Ready(
                    callback: (res) {
                      widget.callback(res);
                      // print("\n\ncalling preview: " +
                      //     res.toString());
                    },
                  )),
        );
      } else {
        verificationTypeList.removeAt(0);
        supportedTypes_list = null;
        typeList = verificationTypeList.asMap();
        isBackside = false;
        isCheck = false;
        if (verificationTypeList.length == 0) {
          if (asyncSDK &&
              !requestedPayload.containsKey('background_checks') &&
              !requestedPayload.containsKey('phone')) {
            check().then((intenet) {
              if (intenet != null && intenet) {
                ApiMethods().apiRequest().then((response) {
                  widget.callback(response);
                });
              } else {
                widget.callback("internet.connection.problem");
              }
              Navigator.pushAndRemoveUntil(
                  context,
                  MaterialPageRoute(builder: (context) => userHome),
                  (route) => false);
              widget.callback("event: request.pending, error: '', reference: " +
                  createdPayloadSDK["reference"]);
            });
          } else {
            Navigator.push(
              context,
              MaterialPageRoute(
                  builder: (context) => Result(
                        callback: (res) {
                          widget.callback(res);
                          // print("\n\ncalling preview: " +
                          //     res.toString());
                        },
                      )),
            );
          }
        } else {
          Navigator.push(
            context,
            MaterialPageRoute(
                builder: (context) => ShowInstructions(
                      callback: (res) {
                        widget.callback(res);
                        // print("\n\ncalling preview: " +
                        //     res.toString());
                      },
                    )),
          );
        }
      }
    }

    Future<bool> _onBackPressed() {
      _current = 10;
      for (int i = 0; i < camCount; i++) {
        Navigator.pop(context);
      }
      camCount = 1;
      // Navigator.push(
      //   context,
      //   MaterialPageRoute(builder: (context) => VideoRecorder(callback: (res) {
      //     widget.callback(res);
      //     // print("\n\ncalling preview: " +
      //     //     res.toString());
      //   },)),
      // );
    }

    // ScreenUtil.init(new BoxConstraints.loose(Size(360, 690)));
    return WillPopScope(
        onWillPop: _onBackPressed,
        child: Scaffold(
          backgroundColor: Colors.white,
          appBar: AppBar(
            backgroundColor: white,
            elevation: 0.0,
            centerTitle: true,
            title: Padding(
              padding: const EdgeInsets.only(
                top: 15,
              ),
              child: Text(
                " Confirmation",
                style: TextStyle(
                  color: Colors.black,
                  fontFamily: 'OpenSans',
                  fontWeight: FontWeight.bold,
                  fontSize: 20.sp,
                ),
              ),
            ),
          ),
          body: Column(
            children: <Widget>[
              Expanded(
                  flex: 1,
                  child: Container(
                    height: 50.h,
                    color: Colors.white,
                    child: Column(
                      children: [
                        Spacer(),
                        typeList[0] == 'face'
                            ? Padding(
                                padding: EdgeInsets.only(
                                  left: 25.w,
                                  right: 15.w,
                                ),
                                child: Container(
                                  alignment: Alignment.topLeft,
                                  child: Text(
                                    "Can you see your full face in the video?",
                                    textAlign: TextAlign.start,
                                    style: new TextStyle(
                                      fontFamily: 'OpenSans',
                                      fontSize: 16.sp,
                                      color: Colors.black,
                                      fontWeight: FontWeight.w600,
                                    ),
                                  ),
                                ),
                              )
                            : typeList[0] == 'document'
                                ? Padding(
                                    padding: EdgeInsets.only(
                                      left: 15.w,
                                      right: 15.w,
                                    ),
                                    child: Container(
                                      alignment: Alignment.topLeft,
                                      child: Text(
                                        "Can you see the entire document clearly in the video?",
                                        textAlign: TextAlign.start,
                                        style: new TextStyle(
                                          fontFamily: 'OpenSans',
                                          fontSize: 16.sp,
                                          color: Colors.black,
                                          fontWeight: FontWeight.w600,
                                        ),
                                      ),
                                    ),
                                  )
                                : typeList[0] == 'document_two'
                                    ? Padding(
                                        padding: EdgeInsets.only(
                                          left: 15.w,
                                          right: 15.w,
                                        ),
                                        child: Container(
                                          alignment: Alignment.topLeft,
                                          child: Text(
                                            "Can you see the entire document clearly in the video?",
                                            textAlign: TextAlign.start,
                                            style: new TextStyle(
                                              fontFamily: 'OpenSans',
                                              fontSize: 16.sp,
                                              color: Colors.black,
                                              fontWeight: FontWeight.w600,
                                            ),
                                          ),
                                        ),
                                      )
                                    : typeList[0] == 'address'
                                        ? Padding(
                                            padding: EdgeInsets.only(
                                              left: 15.w,
                                              right: 15.w,
                                            ),
                                            child: Container(
                                              alignment: Alignment.topLeft,
                                              child: Text(
                                                "Can you see the entire address document clearly in the video?",
                                                textAlign: TextAlign.start,
                                                style: new TextStyle(
                                                  fontFamily: 'OpenSans',
                                                  fontSize: 16.sp,
                                                  color: Colors.black,
                                                  fontWeight: FontWeight.w600,
                                                ),
                                              ),
                                            ),
                                          )
                                        : Padding(
                                            padding: EdgeInsets.only(
                                              left: 15.w,
                                              right: 15.w,
                                            ),
                                            child: Container(
                                              alignment: Alignment.topLeft,
                                              child: Text(
                                                "Can you see your entire face and consent note clearly in the video?",
                                                textAlign: TextAlign.start,
                                                style: new TextStyle(
                                                  fontFamily: 'OpenSans',
                                                  fontSize: 16.sp,
                                                  color: Colors.black,
                                                  fontWeight: FontWeight.w600,
                                                ),
                                              ),
                                            ),
                                          ),
                        Spacer(),
                      ],
                    ),
                  )),
              Expanded(
                flex: 8,
                child: FutureBuilder(
                  future: _initializeVideoPlayerFuture,
                  builder: (context, snapshot) {
                    if (snapshot.connectionState == ConnectionState.done) {
                      return AspectRatio(
                        aspectRatio: _controller.value.aspectRatio,
                        child: GestureDetector(
                          onTap: () async {
                            setState(() {
                              if (_controller.value.isPlaying) {
                                _controller.pause();
                              } else {
                                _controller.play();
                              }
                            });
                          },
                          child: new Stack(
                            children: <Widget>[
                              SizedBox.expand(
                                child: FittedBox(
                                  fit: BoxFit.cover,
                                  child: SizedBox(
                                    width: 0.4.sw,
                                    height: 200.h,
                                    child: cameraDirection == 1
                                        ? Transform(
                                            alignment: Alignment.center,
                                            child: VideoPlayer(_controller),
                                            transform:
                                                Matrix4.rotationY(Math.pi))
                                        : VideoPlayer(_controller),
                                  ),
                                ),
                              ),
                              Container(
                                alignment: Alignment.bottomLeft,
                                padding: EdgeInsets.fromLTRB(0, 1, 1, 1),
                                child: Icon(
                                  _controller.value.isPlaying
                                      ? Icons.pause
                                      : Icons.play_arrow,
                                  size: 35.sp,
                                ),
                              ),
                            ],
                          ),
                        ),
                      );
                    } else {
                      return Center(child: CircularProgressIndicator());
                    }
                  },
                ),
              ),
              Expanded(
                flex: 2,
                child: Column(
                  children: <Widget>[
                    Expanded(
                      flex: 9,
                      child: Row(
                        children: [
                          Expanded(
                            flex: 1,
                            child: Padding(
                              padding: EdgeInsets.only(
                                left: 15.w,
                                right: 5.w,
                              ),
                              child: Container(
                                height: 45.h,
                                child: OutlinedButton(
                                  child: Text(
                                    "Retake",
                                    style: TextStyle(
                                      fontFamily: 'OpenSans',
                                      fontSize: 18.sp,
                                      fontWeight: FontWeight.bold,
                                      color: Colors.black,
                                    ),
                                  ),
                                  style: OutlinedButton.styleFrom(
                                    shape: RoundedRectangleBorder(
                                        borderRadius: BorderRadius.circular(5)),
                                  ),
                                  onPressed: () {
                                    _current = 10;
                                    turn = 3;
                                    // Navigator.pop(context);

                                    // plz = context;

                                    camCount = camCount + 1;
                                    Navigator.push(
                                      context,
                                      MaterialPageRoute(
                                          builder: (context) => VideoRecorder(
                                                callback: (res) {
                                                  widget.callback(res);
                                                  // print("\n\ncalling preview: " +
                                                  //     res.toString());
                                                },
                                              )),
                                    );
                                    // Navigator.pop(context);
                                  },
                                ),
                              ),
                            ),
                          ),
                          Expanded(
                            flex: 1,
                            child: Padding(
                              padding: EdgeInsets.only(
                                left: 0.w,
                                right: 15.w,
                              ),
                              child: Container(
                                height: 45.h,
                                child: OutlinedButton(
                                  style: OutlinedButton.styleFrom(
                                    shape: RoundedRectangleBorder(
                                        borderRadius: BorderRadius.circular(5)),
                                    backgroundColor: Color(0xff2b6ad8),
                                  ),
                                  child: Text(
                                    'Yes, Continue',
                                    style: TextStyle(
                                      fontFamily: 'OpenSans',
                                      fontSize: 18.sp,
                                      fontWeight: FontWeight.bold,
                                      color: Colors.white,
                                    ),
                                  ),
                                  onPressed: () => YesContinue(),
                                ),
                              ),
                            ),
                          )
                        ],
                      ),
                    ),
                    Expanded(
                      flex: 2,
                      child: Padding(
                        padding: EdgeInsets.only(
                          bottom: 14.w,
                        ),
                        child: Container(
                            height: 13.h,
                            child: Image.asset("assets/sdk/footerImage.png", package: 'shufti_test')),
                      ),
                    ),
                  ],
                ),
              )
            ],
          ),
        ));
  }
}
